"""Add EMS objects in common to both SingleZone and MultipleZone."""


def addEMSProgramsBase(self, verboseMode: bool = True):
    """
    Add EMS programs for Base accim.

    Checks if some programs objects are already
    in the model, and otherwise adds them.
    :param verboseMode:
    """
    programlist = ([program.Name for program in self.idf1.idfobjects['EnergyManagementSystem:Program']])

    if 'SetComfTemp' in programlist:
        if verboseMode:
            print('Not added - SetComfTemp Program')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Program',
            Name='SetComfTemp',
            Program_Line_1='if AdapStand == 1',
            Program_Line_2='set ComfTemp = RMOT*0.33+18.8',
            Program_Line_3='elseif AdapStand == 2',
            Program_Line_4='set ComfTemp = PMOT*0.31+17.8',
            Program_Line_5='endif'
            )
        if verboseMode:
            print('Added - SetComfTemp Program')
    #    print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetComfTemp'])

    for zonename in self.zonenames:
        if 'CountHours_'+zonename in programlist:
            if verboseMode:
                print('Not added - CountHours_'+zonename+' Program')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Program',
                Name='CountHours_'+zonename,
                Program_Line_1='if (AdapStand == 1 )',
                Program_Line_2='if (RMOT >= AHSTall) && (RMOT <= ACSTaul)',
                Program_Line_3='if ('+zonename+'_OpT <= ACSTnoTol)',
                Program_Line_4='if ('+zonename+'_OpT >= AHSTnoTol)',
                Program_Line_5='set ComfHours_'+zonename+' = 1*ZoneTimeStep',
                Program_Line_6='set DiscomfAppHotHours_'+zonename+' = 0',
                Program_Line_7='set DiscomfAppColdHours_'+zonename+' = 0',
                Program_Line_8='set DiscomfNonAppHotHours_'+zonename+' = 0',
                Program_Line_9='set DiscomfNonAppColdHours_'+zonename+' = 0',
                Program_Line_10='endif',
                Program_Line_11='elseif ('+zonename+'_OpT > ACSTnoTol)',
                Program_Line_12='set ComfHours_'+zonename+' = 0',
                Program_Line_13='set DiscomfAppHotHours_'+zonename+' = 1*ZoneTimeStep',
                Program_Line_14='set DiscomfAppColdHours_'+zonename+' = 0',
                Program_Line_15='set DiscomfNonAppHotHours_'+zonename+' = 0',
                Program_Line_16='set DiscomfNonAppColdHours_'+zonename+' = 0',
                Program_Line_17='elseif ('+zonename+'_OpT < AHSTnoTol)',
                Program_Line_18='set ComfHours_'+zonename+' = 0',
                Program_Line_19='set DiscomfAppHotHours_'+zonename+' = 0',
                Program_Line_20='set DiscomfAppColdHours_'+zonename+' = 1*ZoneTimeStep',
                Program_Line_21='set DiscomfNonAppHotHours_'+zonename+' = 0',
                Program_Line_22='set DiscomfNonAppColdHours_'+zonename+' = 0',
                Program_Line_23='endif',
                Program_Line_24='elseif (RMOT > ACSTaul)',
                Program_Line_25='set ComfHours_'+zonename+' = 0',
                Program_Line_26='set DiscomfAppHotHours_'+zonename+' = 0',
                Program_Line_27='set DiscomfAppColdHours_'+zonename+' = 0',
                Program_Line_28='set DiscomfNonAppHotHours_'+zonename+' = 1*ZoneTimeStep',
                Program_Line_29='set DiscomfNonAppColdHours_'+zonename+' = 0',
                Program_Line_30='elseif (RMOT < AHSTall)',
                Program_Line_31='set ComfHours_'+zonename+' = 0',
                Program_Line_32='set DiscomfAppHotHours_'+zonename+' = 0',
                Program_Line_33='set DiscomfAppColdHours_'+zonename+' = 0',
                Program_Line_34='set DiscomfNonAppHotHours_'+zonename+' = 0',
                Program_Line_35='set DiscomfNonAppColdHours_'+zonename+' = 1*ZoneTimeStep',
                Program_Line_36='endif',
                Program_Line_37='elseif (AdapStand == 2 )',
                Program_Line_38='if (PMOT >= AHSTall) && (PMOT <= ACSTaul)',
                Program_Line_39='if ('+zonename+'_OpT <= ACSTnoTol)',
                Program_Line_40='if ('+zonename+'_OpT >= AHSTnoTol)',
                Program_Line_41='set ComfHours_'+zonename+' = 1*ZoneTimeStep',
                Program_Line_42='set DiscomfAppHotHours_'+zonename+' = 0',
                Program_Line_43='set DiscomfAppColdHours_'+zonename+' = 0',
                Program_Line_44='set DiscomfNonAppHotHours_'+zonename+' = 0',
                Program_Line_45='set DiscomfNonAppColdHours_'+zonename+' = 0',
                Program_Line_46='endif',
                Program_Line_47='elseif ('+zonename+'_OpT > ACSTnoTol)',
                Program_Line_48='set ComfHours_'+zonename+' = 0',
                Program_Line_49='set DiscomfAppHotHours_'+zonename+' = 1*ZoneTimeStep',
                Program_Line_50='set DiscomfAppColdHours_'+zonename+' = 0',
                Program_Line_51='set DiscomfNonAppHotHours_'+zonename+' = 0',
                Program_Line_52='set DiscomfNonAppColdHours_'+zonename+' = 0',
                Program_Line_53='elseif ('+zonename+'_OpT < AHSTnoTol)',
                Program_Line_54='set ComfHours_'+zonename+' = 0',
                Program_Line_55='set DiscomfAppHotHours_'+zonename+' = 0',
                Program_Line_56='set DiscomfAppColdHours_'+zonename+' = 1*ZoneTimeStep',
                Program_Line_57='set DiscomfNonAppHotHours_'+zonename+' = 0',
                Program_Line_58='set DiscomfNonAppColdHours_'+zonename+' = 0',
                Program_Line_59='endif',
                Program_Line_60='elseif (PMOT > ACSTaul)',
                Program_Line_61='set ComfHours_'+zonename+' = 0',
                Program_Line_62='set DiscomfAppHotHours_'+zonename+' = 0',
                Program_Line_63='set DiscomfAppColdHours_'+zonename+' = 0',
                Program_Line_64='set DiscomfNonAppHotHours_'+zonename+' = 1*ZoneTimeStep',
                Program_Line_65='set DiscomfNonAppColdHours_'+zonename+' = 0',
                Program_Line_66='elseif (PMOT < AHSTall)',
                Program_Line_67='set ComfHours_'+zonename+' = 0',
                Program_Line_68='set DiscomfAppHotHours_'+zonename+' = 0',
                Program_Line_69='set DiscomfAppColdHours_'+zonename+' = 0',
                Program_Line_70='set DiscomfNonAppHotHours_'+zonename+' = 0',
                Program_Line_71='set DiscomfNonAppColdHours_'+zonename+' = 1*ZoneTimeStep',
                Program_Line_72='endif',
                Program_Line_73='endif'
                )
            if verboseMode:
                print('Added - CountHours_'+zonename+' Program')
    #        print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'CountHours_'+zonename])

    if 'SetAppLimits' in programlist:
        if verboseMode:
            print('Not added - SetAppLimits Program')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Program',
            Name='SetAppLimits',
            Program_Line_1='If AdapStand == 1',
            Program_Line_2='set ACSTaul = 30',
            Program_Line_3='set ACSTall = 10',
            Program_Line_4='set AHSTaul = 30',
            Program_Line_5='set AHSTall = 10',
            Program_Line_6='elseif AdapStand == 2',
            Program_Line_7='set ACSTaul = 33.5',
            Program_Line_8='set ACSTall = 10',
            Program_Line_9='set AHSTaul = 33.5',
            Program_Line_10='set AHSTall = 10',
            Program_Line_11='else',
            Program_Line_12='set ACSTaul = 50',
            Program_Line_13='set ACSTall = 50',
            Program_Line_14='set AHSTaul = 50',
            Program_Line_15='set AHSTall = 50',
            Program_Line_16='endif'
            )
        if verboseMode:
            print('Added - SetAppLimits Program')
    #    print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetAppLimits'])

    if 'ApplyCAT' in programlist:
        if verboseMode:
            print('Not added - ApplyCAT Program')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Program',
            Name='ApplyCAT',
            Program_Line_1='if (AdapStand == 1 )',
            Program_Line_2='if (CAT == 1)',
            Program_Line_3='set ACSToffset = 2',
            Program_Line_4='set AHSToffset = -3',
            Program_Line_5='elseif (CAT == 2)',
            Program_Line_6='set ACSToffset = 3',
            Program_Line_7='set AHSToffset = -4',
            Program_Line_8='elseif (CAT == 3)',
            Program_Line_9='set ACSToffset = 4',
            Program_Line_10='set AHSToffset = -5',
            Program_Line_11='endif',
            Program_Line_12='elseif (AdapStand == 2 )',
            Program_Line_13='if (CAT == 90)',
            Program_Line_14='set ACSToffset = 2.5',
            Program_Line_15='set AHSToffset = -2.5',
            Program_Line_16='elseif (CAT == 80)',
            Program_Line_17='set ACSToffset = 3.5',
            Program_Line_18='set AHSToffset = -3.5',
            Program_Line_19='endif',
            Program_Line_20='endif'
            )
        if verboseMode:
            print('Added - ApplyCAT Program')
    #    print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'ApplyCAT'])

    if 'SetAST' in programlist:
        if verboseMode:
            print('Not added - SetAST Program')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Program',
            Name='SetAST',
            Program_Line_1='if (AdapStand == 0) && (CurrentTime < 7)',
            Program_Line_2='set ACST = 27+ACSTtol',
            Program_Line_3='set AHST = 17+AHSTtol',
            Program_Line_4='elseif (AdapStand == 0) && (CurrentTime < 15)',
            Program_Line_5='set ACST = 25',
            Program_Line_6='set AHST = 20+AHSTtol',
            Program_Line_7='elseif (AdapStand == 0) && (CurrentTime < 23)',
            Program_Line_8='set ACST = 25+ACSTtol',
            Program_Line_9='set AHST = 20+AHSTtol',
            Program_Line_10='elseif (AdapStand == 0) && (CurrentTime < 24)',
            Program_Line_11='set ACST = 27+ACSTtol',
            Program_Line_12='set AHST = 17+AHSTtol',
            Program_Line_13='endif',
            Program_Line_14='if (AdapStand == 1) && (ComfMod == 0)',
            Program_Line_15='if (DayOfYear >= 121) && (DayOfYear < 274)',
            Program_Line_16='if (CAT==1)',
            Program_Line_17='set ACST = 25.5+ACSTtol',
            Program_Line_18='elseif (CAT==2)',
            Program_Line_19='set ACST = 26+ACSTtol',
            Program_Line_20='elseif (CAT==3)',
            Program_Line_21='set ACST = 27+ACSTtol',
            Program_Line_22='endif',
            Program_Line_23='else',
            Program_Line_24='if (CAT==1)',
            Program_Line_25='set ACST = 25+ACSTtol',
            Program_Line_26='elseif (CAT==2)',
            Program_Line_27='set ACST = 25+ACSTtol',
            Program_Line_28='elseif (CAT==3)',
            Program_Line_29='set ACST = 25+ACSTtol',
            Program_Line_30='endif',
            Program_Line_31='endif',
            Program_Line_32='endif',
            Program_Line_33='if (AdapStand == 1) && (ComfMod == 0)',
            Program_Line_34='if (DayOfYear >= 121) && (DayOfYear < 274)',
            Program_Line_35='if (CAT==1)',
            Program_Line_36='set AHST = 23.5+AHSTtol',
            Program_Line_37='elseif (CAT==2)',
            Program_Line_38='set AHST = 23+AHSTtol',
            Program_Line_39='elseif (CAT==3)',
            Program_Line_40='set AHST = 22+AHSTtol',
            Program_Line_41='endif',
            Program_Line_42='else',
            Program_Line_43='if (CAT==1)',
            Program_Line_44='set AHST = 21+AHSTtol',
            Program_Line_45='elseif (CAT==2)',
            Program_Line_46='set AHST = 20+AHSTtol',
            Program_Line_47='elseif (CAT==3)',
            Program_Line_48='set AHST = 18+AHSTtol',
            Program_Line_49='endif',
            Program_Line_50='endif',
            Program_Line_51='endif',
            Program_Line_52='if (AdapStand == 1) && (ComfMod == 1) && (RMOT >= ACSTall) && (RMOT <= ACSTaul)',
            Program_Line_53='set ACST = RMOT*0.33+18.8+ACSToffset+ACSTtol',
            Program_Line_54='elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 7)',
            Program_Line_55='set ACST = 27+ACSTtol',
            Program_Line_56='elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 15)',
            Program_Line_57='set ACST = 50',
            Program_Line_58='elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 23)',
            Program_Line_59='set ACST = 25+ACSTtol',
            Program_Line_60='elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 24)',
            Program_Line_61='set ACST = 27+ACSTtol',
            Program_Line_62='endif',
            Program_Line_63='if (AdapStand == 1) && (ComfMod == 1) && (RMOT >= AHSTall) && (RMOT <= AHSTaul)',
            Program_Line_64='set AHST = RMOT*0.33+18.8+AHSToffset+AHSTtol',
            Program_Line_65='elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 7)',
            Program_Line_66='set AHST = 17+AHSTtol',
            Program_Line_67='elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 23)',
            Program_Line_68='set AHST = 20+AHSTtol',
            Program_Line_69='elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 24)',
            Program_Line_70='set AHST = 17+AHSTtol',
            Program_Line_71='endif',
            Program_Line_72='if (AdapStand == 1) && (ComfMod == 2) && (RMOT >= ACSTall) && (RMOT <= ACSTaul)',
            Program_Line_73='set ACST = RMOT*0.33+18.8+ACSToffset+ACSTtol',
            Program_Line_74='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < ACSTall) && (CAT==1)',
            Program_Line_75='set ACST = 25+ACSTtol',
            Program_Line_76='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > ACSTaul) && (CAT==1)',
            Program_Line_77='set ACST = 25.5+ACSTtol',
            Program_Line_78='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < ACSTall) && (CAT==2)',
            Program_Line_79='set ACST = 25+ACSTtol',
            Program_Line_80='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > ACSTaul) && (CAT==2)',
            Program_Line_81='set ACST = 26+ACSTtol',
            Program_Line_82='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < ACSTall) && (CAT==3)',
            Program_Line_83='set ACST = 25+ACSTtol',
            Program_Line_84='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > ACSTaul) && (CAT==3)',
            Program_Line_85='set ACST = 27+ACSTtol',
            Program_Line_86='endif',
            Program_Line_87='if (AdapStand == 1) && (ComfMod == 2) && (RMOT >= AHSTall) && (RMOT <= AHSTaul)',
            Program_Line_88='set AHST = RMOT*0.33+18.8+AHSToffset+AHSTtol',
            Program_Line_89='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < AHSTall) && (CAT==1)',
            Program_Line_90='set AHST = 21+AHSTtol',
            Program_Line_91='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > AHSTaul) && (CAT==1)',
            Program_Line_92='set AHST = 23.5+AHSTtol',
            Program_Line_93='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < AHSTall) && (CAT==2)',
            Program_Line_94='set AHST = 20+AHSTtol',
            Program_Line_95='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > AHSTaul) && (CAT==2)',
            Program_Line_96='set AHST = 23+AHSTtol',
            Program_Line_97='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < AHSTall) && (CAT==3)',
            Program_Line_98='set AHST = 18+AHSTtol',
            Program_Line_99='elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > AHSTaul) && (CAT==3)',
            Program_Line_100='set AHST = 22+AHSTtol',
            Program_Line_101='endif',
            Program_Line_102='if (AdapStand == 1) && (ComfMod == 3) && (RMOT >= ACSTall) && (RMOT <= ACSTaul)',
            Program_Line_103='set ACST = RMOT*0.33+18.8+ACSToffset+ACSTtol',
            Program_Line_104='elseif (AdapStand == 1) && (ComfMod == 3) && (RMOT < ACSTall)',
            Program_Line_105='set ACST = ACSTall*0.33+18.8+ACSToffset+ACSTtol',
            Program_Line_106='elseif (AdapStand == 1) && (ComfMod == 3) && (RMOT > ACSTaul)',
            Program_Line_107='set ACST = ACSTaul*0.33+18.8+ACSToffset+ACSTtol',
            Program_Line_108='endif',
            Program_Line_109='if (AdapStand == 1) && (ComfMod == 3) && (RMOT >= AHSTall) && (RMOT <= AHSTaul)',
            Program_Line_110='set AHST = RMOT*0.33+18.8+AHSToffset+AHSTtol',
            Program_Line_111='elseif (AdapStand == 1) && (ComfMod == 3) && (RMOT < AHSTall)',
            Program_Line_112='set AHST = AHSTall*0.33+18.8+AHSToffset+AHSTtol',
            Program_Line_113='elseif (AdapStand == 1) && (ComfMod == 3) && (RMOT > AHSTaul)',
            Program_Line_114='set AHST = AHSTaul*0.33+18.8+AHSToffset+AHSTtol',
            Program_Line_115='endif',
            Program_Line_116='if (AdapStand == 2) && (ComfMod == 0)',
            Program_Line_117='if (DayOfYear >= 121) && (DayOfYear < 274)',
            Program_Line_118='if (CAT==80)',
            Program_Line_119='set ACST = 26.8+ACSTtol',
            Program_Line_120='elseif (CAT==90)',
            Program_Line_121='set ACST = 25.8+ACSTtol',
            Program_Line_122='endif',
            Program_Line_123='else',
            Program_Line_124='if (CAT==80)',
            Program_Line_125='set ACST = 22.9+ACSTtol',
            Program_Line_126='elseif (CAT==90)',
            Program_Line_127='set ACST = 23.9+ACSTtol',
            Program_Line_128='endif',
            Program_Line_129='endif',
            Program_Line_130='endif',
            Program_Line_131='if (AdapStand == 2) && (ComfMod == 0)',
            Program_Line_132='if (DayOfYear >= 121) && (DayOfYear < 274)',
            Program_Line_133='if (CAT==80)',
            Program_Line_134='set AHST = 24.6+AHSTtol',
            Program_Line_135='elseif (CAT==90)',
            Program_Line_136='set AHST = 23.6+AHSTtol',
            Program_Line_137='endif',
            Program_Line_138='else',
            Program_Line_139='if (CAT==80)',
            Program_Line_140='set AHST = 18.6+AHSTtol',
            Program_Line_141='elseif (CAT==90)',
            Program_Line_142='set AHST = 19.6+AHSTtol',
            Program_Line_143='endif',
            Program_Line_144='endif',
            Program_Line_145='endif',
            Program_Line_146='if (AdapStand == 2) && (ComfMod == 1) && (PMOT >= ACSTall) && (PMOT <= ACSTaul)',
            Program_Line_147='set ACST = PMOT*0.31+17.8+ACSToffset+ACSTtol',
            Program_Line_148='elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 7)',
            Program_Line_149='set ACST = 27+ACSTtol',
            Program_Line_150='elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 15)',
            Program_Line_151='set ACST = 50',
            Program_Line_152='elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 23)',
            Program_Line_153='set ACST = 25+ACSTtol',
            Program_Line_154='elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 24)',
            Program_Line_155='set ACST = 27+ACSTtol',
            Program_Line_156='endif',
            Program_Line_157='if (AdapStand == 2) && (ComfMod == 1) && (PMOT >= AHSTall) && (PMOT <= AHSTaul)',
            Program_Line_158='set AHST = PMOT*0.31+17.8+AHSToffset+AHSTtol',
            Program_Line_159='elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 7)',
            Program_Line_160='set AHST = 17+AHSTtol',
            Program_Line_161='elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 23)',
            Program_Line_162='set AHST = 20+AHSTtol',
            Program_Line_163='elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 24)',
            Program_Line_164='set AHST = 17+AHSTtol',
            Program_Line_165='endif',
            Program_Line_166='if (AdapStand == 2) && (ComfMod == 2) && (PMOT >= ACSTall) && (PMOT <= ACSTaul)',
            Program_Line_167='set ACST = PMOT*0.31+17.8+ACSToffset+ACSTtol',
            Program_Line_168='elseif (AdapStand == 2) && (ComfMod == 2) && (PMOT < ACSTall)',
            Program_Line_169='set ACST = 23.9+ACSTtol',
            Program_Line_170='elseif (AdapStand == 2) && (ComfMod == 2) && (PMOT > ACSTaul)',
            Program_Line_171='set ACST = 26.8+ACSTtol',
            Program_Line_172='endif',
            Program_Line_173='if (AdapStand == 2) && (ComfMod == 2) && (PMOT >= AHSTall) && (PMOT <= AHSTaul)',
            Program_Line_174='set AHST = PMOT*0.31+17.8+AHSToffset+AHSTtol',
            Program_Line_175='elseif (AdapStand == 2) && (ComfMod == 2) && (PMOT < AHSTall)',
            Program_Line_176='set AHST = 19.6+AHSTtol',
            Program_Line_177='elseif (AdapStand == 2) && (ComfMod == 2) && (PMOT > AHSTaul)',
            Program_Line_178='set AHST = 23.6+AHSTtol',
            Program_Line_179='endif',
            Program_Line_180='if (AdapStand == 2) && (ComfMod == 3) && (PMOT >= ACSTall) && (PMOT <= ACSTaul)',
            Program_Line_181='set ACST = PMOT*0.31+17.8+ACSToffset+ACSTtol',
            Program_Line_182='elseif (AdapStand == 2) && (ComfMod == 3) && (PMOT < ACSTall)',
            Program_Line_183='set ACST = ACSTall*0.31+17.8+ACSToffset+ACSTtol',
            Program_Line_184='elseif (AdapStand == 2) && (ComfMod == 3) && (PMOT > ACSTaul)',
            Program_Line_185='set ACST = ACSTaul*0.31+17.8+ACSToffset+ACSTtol',
            Program_Line_186='endif',
            Program_Line_187='if (AdapStand == 2) && (ComfMod == 3) && (PMOT >= AHSTall) && (PMOT <= AHSTaul)',
            Program_Line_188='set AHST = PMOT*0.31+17.8+AHSToffset+AHSTtol',
            Program_Line_189='elseif (AdapStand == 2) && (ComfMod == 3) && (PMOT < AHSTall)',
            Program_Line_190='set AHST = AHSTall*0.31+17.8+AHSToffset+AHSTtol',
            Program_Line_191='elseif (AdapStand == 2) && (ComfMod == 3) && (PMOT > AHSTaul)',
            Program_Line_192='set AHST = AHSTaul*0.31+17.8+AHSToffset+AHSTtol',
            Program_Line_193='endif'
            )
        if verboseMode:
            print('Added - SetAST Program')
    #    print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetAST'])

    if 'SetASTnoTol' in programlist:
        if verboseMode:
            print('Not added - SetASTnoTol Program')
    else:
        self.idf1.newidfobject(
            'EnergyManagementSystem:Program',
            Name='SetASTnoTol',
            Program_Line_1='set ACSTnoTol = ACST-ACSTtol',
            Program_Line_2='set AHSTnoTol = AHST-AHSTtol'
            )
        if verboseMode:
            print('Added - SetASTnoTol Program')
    #    print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetASTnoTol'])

    for zonename in self.zonenames:
        if 'CountHoursNoApp_'+zonename in programlist:
            if verboseMode:
                print('Not added - CountHoursNoApp_'+zonename+' Program')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:Program',
                Name='CountHoursNoApp_'+zonename,
                Program_Line_1='if ('+zonename+'_OpT <= ACSTnoTol)',
                Program_Line_2='if ('+zonename+'_OpT >= AHSTnoTol)',
                Program_Line_3='set ComfHoursNoApp_'+zonename+'  = 1*ZoneTimeStep',
                Program_Line_4='else',
                Program_Line_5='set ComfHoursNoApp_'+zonename+' = 0',
                Program_Line_6='endif',
                Program_Line_7='else',
                Program_Line_8='set ComfHoursNoApp_'+zonename+' = 0',
                Program_Line_9='endif'
                )
            if verboseMode:
                print('Added - CountHoursNoApp_'+zonename+' Program')
    #        print([program for program in self.idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'CountHoursNoApp_'+zonename])

    del programlist


def addEMSPCMBase(self, verboseMode: bool = True):
    """
    Add EMS program calling managers for Base accim.

    Checks if some EMS program calling manager objects are already
    in the model, and otherwise adds them.
    """
    programlist = ([program.Name for program in self.idf1.idfobjects['EnergyManagementSystem:Program']])
    pcmlist = ([pcm.Name for pcm in self.idf1.idfobjects['EnergyManagementSystem:ProgramCallingManager']])

    for i in programlist:
        if i in pcmlist:
            if verboseMode:
                print('Not added - '+i+' Program Calling Manager')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:ProgramCallingManager',
                Name=i,
                EnergyPlus_Model_Calling_Point="BeginTimestepBeforePredictor",
                Program_Name_1=i
                )
            if verboseMode:
                print('Added - '+i+' Program Calling Manager')
    #        print([program for program in self.idf1.idfobjects['EnergyManagementSystem:ProgramCallingManager'] if program.Name == i])

    del programlist, pcmlist


def addEMSOutputVariableBase(self, verboseMode: bool = True):
    """Add EMS output variables for Base accim.

    Checks if some EMS output variables objects are already
    in the model, and otherwise adds them.
    """
    EMSOutputVariableComfTemp_dict = {
        'Comfort Temperature': 'ComfTemp',
        'Adaptive Cooling Setpoint Temperature': 'ACST',
        'Adaptive Heating Setpoint Temperature': 'AHST',
        'Adaptive Cooling Setpoint Temperature_No Tolerance': 'ACSTnoTol',
        'Adaptive Heating Setpoint Temperature_No Tolerance': 'AHSTnoTol',
        }

    outputvariablelist = ([program.Name for program in self.idf1.idfobjects['EnergyManagementSystem:OutputVariable']])

    for i in EMSOutputVariableComfTemp_dict:
        if i in outputvariablelist:
            if verboseMode:
                print('Not added - '+i+' Output Variable')
        else:
            self.idf1.newidfobject(
                'EnergyManagementSystem:OutputVariable',
                Name=i,
                EMS_Variable_Name=EMSOutputVariableComfTemp_dict[i],
                Type_of_Data_in_Variable='Averaged',
                Update_Frequency='ZoneTimestep',
                EMS_Program_or_Subroutine_Name='',
                Units='C'
                )
            if verboseMode:
                print('Added - '+i+' Output Variable')
            # print([outputvariable for outputvariable in self.idf1.idfobjects['EnergyManagementSystem:OutputVariable'] if outputvariable.Name == i])

    EMSOutputVariableComfHours_dict = {
        'Comfortable Hours_No Applicability': 'ComfHoursNoApp',
        'Comfortable Hours': 'ComfHours',
        'Discomfortable Applicable Hot Hours': 'DiscomfAppHotHours',
        'Discomfortable Applicable Cold Hours': 'DiscomfAppColdHours',
        'Discomfortable Non Applicable Hot Hours': 'DiscomfNonAppHotHours',
        'Discomfortable Non Applicable Cold Hours': 'DiscomfNonAppColdHours',
        }

    for i in EMSOutputVariableComfHours_dict:
        for zonename in self.zonenames:
            if i+'_'+zonename+' (summed)' in outputvariablelist:
                if verboseMode:
                    print('Not added - '+i+'_'
                          + zonename + ' (summed) Output Variable')
            else:
                self.idf1.newidfobject(
                    'EnergyManagementSystem:OutputVariable',
                    Name=i + '_' + zonename + ' (summed)',
                    EMS_Variable_Name=EMSOutputVariableComfHours_dict[i]+'_'
                    + zonename,
                    Type_of_Data_in_Variable='Summed',
                    Update_Frequency='ZoneTimestep',
                    EMS_Program_or_Subroutine_Name='',
                    Units='H'
                    )
                if verboseMode:
                    print('Added - '+i+'_'
                          + zonename + ' (summed) Output Variable')
                # print([outputvariable for outputvariable in self.idf1.idfobjects['EnergyManagementSystem:OutputVariable'] if outputvariable.Name == i+'_'+zonename+' (summed)'])

    del outputvariablelist


def addOutputVariablesTimestep(self, verboseMode: bool = True):
    """
    Add Output:Variable objects in timestep frequency.

    No need for further description.
    """
    fulloutputlist = ([output for output in self.idf1.idfobjects['Output:Variable']])
    # print(fulloutputlist)

    outputlist = ([output.Variable_Name for output in self.idf1.idfobjects['Output:Variable']])
    # print(outputlist)

    for i in range(len(outputlist)):
        self.idf1.newidfobject(
            'Output:Variable',
            Key_Value=fulloutputlist[i].Key_Value,
            Variable_Name=fulloutputlist[i].Variable_Name,
            Reporting_Frequency='Timestep',
            Schedule_Name=fulloutputlist[i].Schedule_Name
            )
        if verboseMode:
            print('Added - '+fulloutputlist[i].Variable_Name+' Output:Variable Timestep data')

    # print([output for output in self.idf1.idfobjects['Output:Variable'] if output.Reporting_Frequency == 'Timestep'])

    del fulloutputlist, outputlist


def addSimplifiedOutputVariables(self, verboseMode: bool = True):
    """
    Add simplified Output:Variable objects for accim.

    Remove all outputs and add only VFR outdoor unit consumption
    and operative temperature.
    """
    EnvironmentalImpactFactorslist = ([output for output in self.idf1.idfobjects['Output:EnvironmentalImpactFactors']])
    for i in range(len(EnvironmentalImpactFactorslist)):
        firstEnvironmentalImpactFactor = self.idf1.idfobjects['Output:EnvironmentalImpactFactors'][-1]
        self.idf1.removeidfobject(firstEnvironmentalImpactFactor)

    outputmeterlist = ([output for output in self.idf1.idfobjects['Output:Meter']])
    for i in range(len(outputmeterlist)):
        firstoutputmeter = self.idf1.idfobjects['Output:Meter'][-1]
        self.idf1.removeidfobject(firstoutputmeter)

    alloutputs = ([output for output in self.idf1.idfobjects['Output:Variable']])
    for i in range(len(alloutputs)):
        firstoutput = self.idf1.idfobjects['Output:Variable'][-1]
        self.idf1.removeidfobject(firstoutput)

    # del EnvironmentalImpactFactorslist,firstEnvironmentalImpactFactor, outputmeterlist, firstoutputmeter, alloutputs, firstoutput

    addittionaloutputs = ['Zone Thermostat Operative Temperature',
                          'VRF Heat Pump Cooling Electricity Energy',
                          'VRF Heat Pump Heating Electricity Energy',
                          'Facility Total HVAC Electric Demand Power',
                          'Facility Total HVAC Electricity Demand Rate'
                          ]

    for addittionaloutput in addittionaloutputs:
        self.idf1.newidfobject(
            'Output:Variable',
            Key_Value='*',
            Variable_Name=addittionaloutput,
            Reporting_Frequency='Hourly',
            Schedule_Name=''
            )
        if verboseMode:
            print('Added - '+addittionaloutput+' Output:Variable data')

    del addittionaloutputs



"""
Run any of the functions below to add the accis.

These functions transform fixed setpoint temperature
building energy models into adaptive setpoint temperature energy models
by adding the Adaptive Comfort Control Implementation Script (ACCIS)
"""


def addAccisSingleZoneEp91():
    """Add SingleZone accis."""
    import accim.sim.accim_Main as accim_Main
    from os import listdir
    # todo see why it includes output idfs from accim (files with _pymod in the name), check in all accis functions

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_SingleZone_Ep91(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addForscriptSchSingleZone()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListSingleZone()
        z.addEMSSensorsSingleZone()
        z.addEMSActuatorsSingleZone()
        z.addEMSProgramsSingleZone()
        z.addEMSPCMBase()
        z.addOutputVariablesSingleZone()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataSingleZone()
    z.genIDFSingleZone()


def addAccisSingleZoneEp94():
    """Add SingleZone accis."""
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_SingleZone_Ep94(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addForscriptSchSingleZone()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListSingleZone()
        z.addEMSSensorsSingleZone()
        z.addEMSActuatorsSingleZone()
        z.addEMSProgramsSingleZone()
        z.addEMSPCMBase()
        z.addOutputVariablesSingleZone()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataSingleZone()
    z.genIDFSingleZone()


def addAccisSingleZoneTimestepEp91():
    """Add SingleZone accis with timestep output data frecuency."""
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_SingleZone_Ep91(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addForscriptSchSingleZone()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListSingleZone()
        z.addEMSSensorsSingleZone()
        z.addEMSActuatorsSingleZone()
        z.addEMSProgramsSingleZone()
        z.addEMSPCMBase()
        z.addOutputVariablesSingleZone()
        z.addOutputVariablesTimestep()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataSingleZone()
    z.genIDFSingleZone()


def addAccisSingleZoneTimestepEp94():
    """Add SingleZone accis with timestep output data frecuency."""
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_SingleZone_Ep94(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addForscriptSchSingleZone()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListSingleZone()
        z.addEMSSensorsSingleZone()
        z.addEMSActuatorsSingleZone()
        z.addEMSProgramsSingleZone()
        z.addEMSPCMBase()
        z.addOutputVariablesSingleZone()
        z.addOutputVariablesTimestep()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataSingleZone()
    z.genIDFSingleZone()


def addAccisSingleZoneSimplifiedEp91():
    """Add SingleZone accis with simplified outputs."""
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_SingleZone_Ep91(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addForscriptSchSingleZone()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListSingleZone()
        z.addEMSSensorsSingleZone()
        z.addEMSActuatorsSingleZone()
        z.addEMSProgramsSingleZone()
        z.addEMSPCMBase()
        z.addSimplifiedOutputVariables()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataSingleZone()
    z.genIDFSingleZone()


def addAccisSingleZoneSimplifiedEp94():
    """Add SingleZone accis with simplified outputs."""
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_SingleZone_Ep94(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addForscriptSchSingleZone()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListSingleZone()
        z.addEMSSensorsSingleZone()
        z.addEMSActuatorsSingleZone()
        z.addEMSProgramsSingleZone()
        z.addEMSPCMBase()
        z.addSimplifiedOutputVariables()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataSingleZone()
    z.genIDFSingleZone()


def addAccisMultipleZoneEp91():
    """Add MultipleZone accis."""
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_MultipleZone_Ep91(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addMultipleZoneSch()
        z.addCurveObj()
        z.addDetHVACobjEp91()
        z.addForscriptSchMultipleZone()
        z.checkVentIsOn()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListMultipleZone()
        z.addEMSSensorsMultipleZone()
        z.addEMSActuatorsMultipleZone()
        z.addEMSProgramsMultipleZone()
        z.addEMSPCMBase()
        z.addEMSOutputVariableMultipleZone()
        z.addOutputVariablesMultipleZone()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataMultipleZone()
    z.genIDFMultipleZone()


def addAccisMultipleZoneEp94():
    """Add MultipleZone accis."""
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_MultipleZone_Ep94(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addMultipleZoneSch()
        z.addCurveObj()
        z.addDetHVACobjEp94()
        z.addForscriptSchMultipleZone()
        z.checkVentIsOn()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListMultipleZone()
        z.addEMSSensorsMultipleZone()
        z.addEMSActuatorsMultipleZone()
        z.addEMSProgramsMultipleZone()
        z.addEMSPCMBase()
        z.addEMSOutputVariableMultipleZone()
        z.addOutputVariablesMultipleZone()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataMultipleZone()
    z.genIDFMultipleZone()


def addAccisMultipleZoneTimestepEp91():
    """Add MultipleZone accis with timestep output data frecuency."""
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_MultipleZone_Ep91(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addMultipleZoneSch()
        z.addCurveObj()
        z.addDetHVACobjEp91()
        z.addForscriptSchMultipleZone()
        z.checkVentIsOn()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListMultipleZone()
        z.addEMSSensorsMultipleZone()
        z.addEMSActuatorsMultipleZone()
        z.addEMSProgramsMultipleZone()
        z.addEMSPCMBase()
        z.addEMSOutputVariableMultipleZone()
        z.addOutputVariablesMultipleZone()
        z.addOutputVariablesTimestep()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataMultipleZone()
    z.genIDFMultipleZone()


def addAccisMultipleZoneTimestepEp94():
    """Add MultipleZone accis with timestep output data frecuency."""
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_MultipleZone_Ep94(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addMultipleZoneSch()
        z.addCurveObj()
        z.addDetHVACobjEp94()
        z.addForscriptSchMultipleZone()
        z.checkVentIsOn()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListMultipleZone()
        z.addEMSSensorsMultipleZone()
        z.addEMSActuatorsMultipleZone()
        z.addEMSProgramsMultipleZone()
        z.addEMSPCMBase()
        z.addEMSOutputVariableMultipleZone()
        z.addOutputVariablesMultipleZone()
        z.addOutputVariablesTimestep()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataMultipleZone()
    z.genIDFMultipleZone()


def addAccisMultipleZoneSimplifiedEp91():
    """Add MultipleZone accis with simplified output."""
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_MultipleZone_Ep91(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addMultipleZoneSch()
        z.addCurveObj()
        z.addDetHVACobjEp91()
        z.addForscriptSchMultipleZone()
        z.checkVentIsOn()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListMultipleZone()
        z.addEMSSensorsMultipleZone()
        z.addEMSActuatorsMultipleZone()
        z.addEMSProgramsMultipleZone()
        z.addEMSPCMBase()
        z.addEMSOutputVariableMultipleZone()
        z.addSimplifiedOutputVariables()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataMultipleZone()
    z.genIDFMultipleZone()


def addAccisMultipleZoneSimplifiedEp94():
    """Add MultipleZone accis with simplified output."""
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not file.endswith('_pymod.idf')])

    filelist = ([file.split('.idf')[0] for file in filelist])

    for file in filelist:
        z = accim_Main.accimobj_MultipleZone_Ep94(file)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        z.addMultipleZoneSch()
        z.addCurveObj()
        z.addDetHVACobjEp94()
        z.addForscriptSchMultipleZone()
        z.checkVentIsOn()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        z.addGlobVarListMultipleZone()
        z.addEMSSensorsMultipleZone()
        z.addEMSActuatorsMultipleZone()
        z.addEMSProgramsMultipleZone()
        z.addEMSPCMBase()
        z.addEMSOutputVariableMultipleZone()
        z.addSimplifiedOutputVariables()

        z.saveaccim()

    z = accim_Main.accimobj()
    z.inputdataMultipleZone()
    z.genIDFMultipleZone()


def addAccis(
        ScriptType: str = None,
        Outputs: str = None,
        EnergyPlus_version: str = None,
        AdapStand: any = None,
        CAT: any = None,
        ComfMod: any = None,
        HVACmode: any = None,
        VentCtrl: any = None,
        VSToffset: any = [0],
        MinOToffset: any = [50],
        MaxWindSpeed: any = [50],
        ASTtol_start: float = 0.1,
        ASTtol_end_input: float = 0.1,
        ASTtol_steps: float = 0.1,
        NameSuffix: str = ''
        ):
    """
    Parameters
    :param ScriptType: The default is 'MultipleZones'. Can be 'MultipleZones'or
        'mz', or 'SingleZone' or 'sz'.
    :param Outputs: The default is 'Standard'. Can be 'Standard',
        'Simplified' or 'Timestep'.
    :param EnergyPlus_version: The default is 'Ep94'. Can be 'Ep91' or 'Ep94'.
    :param AdapStand: The default is None.(0 = CTE; 1 = EN16798-1; 2 = ASHRAE 55)
    :param CAT: The default is None.(1 = CAT I; 2 = CAT II; 3 = CAT III; 80 = 80% ACCEPT; 90 = 90% ACCEPT)
    :param ComfMod: The default is None.(0 = Static; 1 = OUT-CTE; 2 = OUT-SEN16798/SASHRAE55; 3 = OUT-AEN16798/AASHRAE55)
    :param HVACmode: The default is None.(0 = Fully Air-conditioned; 1 = Naturally ventilated; 2 = Mixed Mode)
    :param VentCtrl: (0 = Ventilates above neutral temperature; 1 = Ventilates above upper comfort limit)
    :param VSToffset: Please refer to documentation.
    :param MinOToffset: Please refer to documentation.
    :param MaxWindSpeed: Please refer to documentation.
    :param ASTtol_start: Please refer to documentation.
    :param ASTtol_end_input: Please refer to documentation.
    :param ASTtol_steps: Please refer to documentation.
    :param NameSuffix: Please refer to documentation.

    Exceptions
            DESCRIPTION. EnergyPlus version not supported.
        Only works for EnergyPlus 9.1 (enter Ep91) and
        EnergyPlus 9.4 (enter Ep94) versions.

    :return:
    """
    import accim.sim.accim_Main as accim_Main

    from os import listdir

    filelist = ([file for file in listdir() if file.endswith('.idf')
                 and not '_pymod' in file])

    filelist = ([file.split('.idf')[0] for file in filelist])

    objArgsDef = (
        ScriptType is None,
        Outputs is None,
        EnergyPlus_version is None
    )

    fullScriptTypeList = ['MultipleZone',
                          'multiplezone',
                          'mz',
                          'SingleZone',
                          'singlezone',
                          'sz'
    ]

    fullOutputsList = [
        'Standard',
        'standard',
        'Simplified',
        'simplified',
        'Timestep',
        'timestep'
    ]

    fullEPversionsList = [
        'Ep91',
        'ep91',
        'Ep94',
        'ep94'
    ]

    if all(objArgsDef):
        ScriptType = input("Enter the ScriptType (MultipleZone or mz, or SingleZone or sz): ")
        while ScriptType not in fullScriptTypeList:
            ScriptType = input("ScriptType was not correct. Please, enter the ScriptType (MultipleZone or mz, or SingleZone or sz): ")
        Outputs = input("Enter the Output (Standard, Simplified or Timestep): ")
        while Outputs not in fullOutputsList:
            Outputs = input("Output was not correct. Please, enter the Output (Standard, Simplified or Timestep): ")
        EnergyPlus_version = input("Enter the EnergyPlus version (Ep91 or Ep94): ")
        while EnergyPlus_version not in fullEPversionsList:
            EnergyPlus_version = input("EnergyPlus version was not correct. Please, enter the EnergyPlus version (Ep91 or Ep94): ")

    for file in filelist:
        print('''\n=======================START OF PROCESS=======================\n''')
        print('Starting with file:')
        print(file)
        z = accim_Main.accimInstance(filename_temp=file, ScriptType=ScriptType, EnergyPlus_version=EnergyPlus_version)

        z.setComfFieldsPeople()
        z.addOpTempTherm()
        z.addBaseSchedules()
        z.setAvailSchOn()

        if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
            z.addMultipleZoneSch()
            z.addCurveObj()
            if EnergyPlus_version.lower() == 'ep94':
                z.addDetHVACobjEp94()
            elif EnergyPlus_version.lower() == 'ep91':
                z.addDetHVACobjEp91()
            z.checkVentIsOn()
            z.addForscriptSchMultipleZone()
        elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
            z.addForscriptSchSingleZone()

        z.addEMSProgramsBase()
        z.addEMSOutputVariableBase()

        if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
            z.addGlobVarListMultipleZone()
            z.addEMSSensorsMultipleZone()
            z.addEMSActuatorsMultipleZone()
            z.addEMSProgramsMultipleZone()
        elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
            z.addGlobVarListSingleZone()
            z.addEMSSensorsSingleZone()
            z.addEMSActuatorsSingleZone()
            z.addEMSProgramsSingleZone()

        z.addEMSPCMBase()

        if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
            z.addEMSOutputVariableMultipleZone()
            if Outputs.lower() == 'Simplified'.lower():
                z.addSimplifiedOutputVariables()
            elif Outputs.lower() == 'Standard'.lower():
                z.addOutputVariablesMultipleZone()
        elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
            if Outputs.lower() == 'Simplified'.lower():
                z.addSimplifiedOutputVariables()
            elif Outputs.lower() == 'Standard'.lower():
                z.addOutputVariablesSingleZone()
        if Outputs.lower() == 'Timestep'.lower():
            z.addOutputVariablesTimestep()

        z.saveaccim()
        print('Ending with file:')
        print(file)
        print('''\n=======================END OF PROCESS=======================\n''')

    z = accim_Main.accimobj()

    arguments = (
        AdapStand is None,
        CAT is None,
        ComfMod is None,
        HVACmode is None,
        VentCtrl is None,
        VSToffset == [0],
        MinOToffset == [50],
        MaxWindSpeed == [50],
        ASTtol_start == 0.1,
        ASTtol_end_input == 0.1,
        ASTtol_steps == 0.1
        )

    if ScriptType.lower() == 'MultipleZones'.lower() or ScriptType.lower() == 'mz':
        if all(arguments):
            z.inputdataMultipleZone()
            z.genIDFMultipleZone()
        else:
            z.genIDFMultipleZone(
                AdapStand,
                CAT,
                ComfMod,
                HVACmode,
                VentCtrl,
                VSToffset,
                MinOToffset,
                MaxWindSpeed,
                ASTtol_start,
                ASTtol_end_input,
                ASTtol_steps,
                NameSuffix
                )
    elif ScriptType.lower() == 'SingleZone'.lower() or ScriptType.lower() == 'sz':
        if all(arguments):
            z.inputdataSingleZone()
            z.genIDFSingleZone()
        else:
            z.genIDFSingleZone(
                AdapStand,
                CAT,
                ComfMod,
                ASTtol_start,
                ASTtol_end_input,
                ASTtol_steps,
                NameSuffix
                )



"""Generate IDFs."""


def inputdataSingleZone(self):
    """Input data for IDF generation in SingleZone."""
    fullAdapStandList = [0, 1, 2]
    self.AdapStand_List = list(int(num) for num in input("Enter the Adaptive Standard numbers separated by space (0 = CTE; 1 = EN16798-1; 2 = ASHRAE 55): ").split())
    while len(self.AdapStand_List) == 0 or not all(elem in fullAdapStandList for elem in self.AdapStand_List):
        print('          Adaptive Standard numbers are not correct. Please enter the numbers again.')
        self.AdapStand_List = list(int(num) for num in input("     Enter the Adaptive Standard numbers separated by space: ").split())
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.AdapStand_List = list(int(num) for num in input("     Enter the Adaptive Standard numbers separated by space: ").split())
        while len(self.AdapStand_List) == 0 or not all(elem in fullAdapStandList for elem in self.AdapStand_List):
            print('          Adaptive Standard numbers are not correct. Please enter the numbers again.')
            self.AdapStand_List = list(int(num) for num in input("     Enter the Adaptive Standard numbers separated by space: ").split())

    fullCATlist = [1, 2, 3, 80, 90]
    self.CAT_List = list(int(num) for num in input("Enter the Category numbers separated by space (1 = CAT I; 2 = CAT II; 3 = CAT III; 80 = 80% ACCEPT; 90 = 90% ACCEPT): ").split())
    while len(self.CAT_List) == 0 or not all(elem in fullCATlist for elem in self.CAT_List):
        print('          Category numbers are not correct. Please enter the numbers again.')
        self.CAT_List = list(int(num) for num in input("Enter the Category numbers separated by space: ").split())
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.CAT_List = list(int(num) for num in input("     Enter the Category numbers separated by space: ").split())
        while len(self.CAT_List) == 0 or not all(elem in fullCATlist for elem in self.CAT_List):
            print('          Category numbers are not correct. Please enter the numbers again.')
            self.CAT_List = list(int(num) for num in input("Enter the Category numbers separated by space: ").split())

    fullComfModList = [0, 1, 2, 3]
    self.ComfMod_List = list(int(num) for num in input("Enter the Comfort Mode numbers separated by space (0 = Static; 1 = OUT-CTE; 2 = OUT-SEN16798/SASHRAE55; 3 = OUT-AEN16798/AASHRAE55): ").split())
    while len(self.ComfMod_List) == 0 or not all(elem in fullComfModList for elem in self.ComfMod_List):
        print('          Comfort Mode numbers are not correct. Please enter the numbers again.')
        self.ComfMod_List = list(int(num) for num in input("     Enter the Comfort Mode numbers separated by space: ").split())
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.ComfMod_List = list(int(num) for num in input("     Enter the Comfort Mode numbers separated by space: ").split())
        while len(self.ComfMod_List) == 0 or not all(elem in fullComfModList for elem in self.ComfMod_List):
            print('          Comfort Mode numbers are not correct. Please enter the numbers again.')
            self.ComfMod_List = list(int(num) for num in input("     Enter the Comfort Mode numbers separated by space: ").split())

    try:
        self.ASTtol_value_from = float(input('Enter the ASTtol value from (if omitted, will be 0.1): '))
    except ValueError:
        self.ASTtol_value_from = float(0.1)
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        try:
            self.ASTtol_value_from = float(input('     Enter the ASTtol value from (if omitted, will be 0.1): '))
        except ValueError:
            self.ASTtol_value_from = float(0.1)

    try:
        self.ASTtol_value_to_input = float(input('Enter the ASTtol value to (if omitted, will be 0.1): '))
    except ValueError:
        self.ASTtol_value_to_input = float(0.1)
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        try:
            self.ASTtol_value_to_input = float(input('     Enter the ASTtol value to (if omitted, will be 0.1): '))
        except ValueError:
            self.ASTtol_value_to_input = float(0.1)

    try:
        self.ASTtol_value_steps = float(input('Enter the ASTtol value steps (if omitted, will be 0.1): '))
    except ValueError:
        self.ASTtol_value_steps = float(0.1)
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        try:
            self.ASTtol_value_steps = float(input('     Enter the ASTtol value steps (if omitted, will be 0.1): '))
        except ValueError:
            self.ASTtol_value_steps = float(0.1)



def inputdataMultipleZone(self):
    """Input data for IDF generation in MultipleZone."""
    fullAdapStandList = [0, 1, 2]
    self.AdapStand_List = list(int(num) for num in input("Enter the Adaptive Standard numbers separated by space (0 = CTE; 1 = EN16798-1; 2 = ASHRAE 55): ").split())
    while len(self.AdapStand_List) == 0 or not all(elem in fullAdapStandList for elem in self.AdapStand_List):
        print('          Adaptive Standard numbers are not correct. Please enter the numbers again.')
        self.AdapStand_List = list(int(num) for num in input("     Enter the Adaptive Standard numbers separated by space: ").split())
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.AdapStand_List = list(int(num) for num in input("     Enter the Adaptive Standard numbers separated by space: ").split())
        while len(self.AdapStand_List) == 0 or not all(elem in fullAdapStandList for elem in self.AdapStand_List):
            print('          Adaptive Standard numbers are not correct. Please enter the numbers again.')
            self.AdapStand_List = list(int(num) for num in input("     Enter the Adaptive Standard numbers separated by space: ").split())

    fullCATlist = [1, 2, 3, 80, 90]
    self.CAT_List = list(int(num) for num in input("Enter the Category numbers separated by space (1 = CAT I; 2 = CAT II; 3 = CAT III; 80 = 80% ACCEPT; 90 = 90% ACCEPT): ").split())
    while len(self.CAT_List) == 0 or not all(elem in fullCATlist for elem in self.CAT_List):
        print('          Category numbers are not correct. Please enter the numbers again.')
        self.CAT_List = list(int(num) for num in input("Enter the Category numbers separated by space: ").split())
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.CAT_List = list(int(num) for num in input("     Enter the Category numbers separated by space: ").split())
        while len(self.CAT_List) == 0 or not all(elem in fullCATlist for elem in self.CAT_List):
            print('          Category numbers are not correct. Please enter the numbers again.')
            self.CAT_List = list(int(num) for num in input("Enter the Category numbers separated by space: ").split())

    fullComfModList = [0, 1, 2, 3]
    self.ComfMod_List = list(int(num) for num in input("Enter the Comfort Mode numbers separated by space (0 = Static; 1 = OUT-CTE; 2 = OUT-SEN16798/SASHRAE55; 3 = OUT-AEN16798/AASHRAE55): ").split())
    while len(self.ComfMod_List) == 0 or not all(elem in fullComfModList for elem in self.ComfMod_List):
        print('          Comfort Mode numbers are not correct. Please enter the numbers again.')
        self.ComfMod_List = list(int(num) for num in input("     Enter the Comfort Mode numbers separated by space: ").split())
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.ComfMod_List = list(int(num) for num in input("     Enter the Comfort Mode numbers separated by space: ").split())
        while len(self.ComfMod_List) == 0 or not all(elem in fullComfModList for elem in self.ComfMod_List):
            print('          Comfort Mode numbers are not correct. Please enter the numbers again.')
            self.ComfMod_List = list(int(num) for num in input("     Enter the Comfort Mode numbers separated by space: ").split())

    fullHVACmodeList = [0, 1, 2]
    self.HVACmode_List = list(int(num) for num in input("Enter the HVAC Mode numbers separated by space (0 = Fully Air-conditioned; 1 = Naturally ventilated; 2 = Mixed Mode): ").split())
    while len(self.HVACmode_List) == 0 or not all(elem in fullHVACmodeList for elem in self.HVACmode_List):
        print('          HVACmode numbers are not correct. Please enter the numbers again.')
        self.HVACmode_List = list(int(num) for num in input("     Enter the HVACmode numbers separated by space: ").split())
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.HVACmode_List = list(int(num) for num in input("     Enter the HVACmode numbers separated by space: ").split())
        while len(self.HVACmode_List) == 0 or not all(elem in fullHVACmodeList for elem in self.HVACmode_List):
            print('          HVACmode numbers are not correct. Please enter the numbers again.')
            self.HVACmode_List = list(int(num) for num in input("     Enter the HVACmode numbers separated by space: ").split())

    fullVentCtrlList = [0, 1]
    self.VentCtrl_List = list(int(num) for num in input("Enter the Ventilation Control numbers separated by space (0 = Ventilates above neutral temperature; 1 = Ventilates above upper comfort limit): ").split())
    while len(self.VentCtrl_List) == 0 or not all(elem in fullVentCtrlList for elem in self.VentCtrl_List):
        print('          Ventilation Control numbers are not correct. Please enter the numbers again.')
        self.VentCtrl_List = list(int(num) for num in input("     Enter the Ventilation Control numbers separated by space: ").split())
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.VentCtrl_List = list(int(num) for num in input("     Enter the Ventilation Control numbers separated by space: ").split())
        while len(self.VentCtrl_List) == 0 or not all(elem in fullVentCtrlList for elem in self.VentCtrl_List):
            print('          Ventilation Control numbers are not correct. Please enter the numbers again.')
            self.VentCtrl_List = list(int(num) for num in input("     Enter the Ventilation Control numbers separated by space: ").split())

    self.VSToffset_List = list(float(num) for num in input("Enter the VSToffset numbers separated by space (if omitted, will be 0): ").split())
    if len(self.VSToffset_List) == 0:
        self.VSToffset_List = [float(0)]
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.VSToffset_List = list(float(num) for num in input("     Enter the VSToffset numbers separated by space (if omitted, will be 0): ").split())
        if len(self.VSToffset_List) == 0:
            self.VSToffset_List = [float(0)]

    self.MinOToffset_List = list(float(num) for num in input("Enter the MinOToffset numbers separated by space (if omitted, will be 50): ").split())
    if len(self.MinOToffset_List) == 0:
        self.MinOToffset_List = [float(50)]
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.MinOToffset_List = list(float(num) for num in input("     Enter the MinOToffset numbers separated by space (if omitted, will be 50): ").split())
        if len(self.MinOToffset_List) == 0:
            self.MinOToffset_List = [float(50)]

    self.MaxWindSpeed_List = list(float(num) for num in input("Enter the MaxWindSpeed numbers separated by space (if omitted, will be 50): ").split())
    if len(self.MaxWindSpeed_List) == 0:
        self.MaxWindSpeed_List = [float(50)]
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        self.MaxWindSpeed_List = list(float(num) for num in input("     Enter the MaxWindSpeed numbers separated by space (if omitted, will be 50): ").split())
        if len(self.MaxWindSpeed_List) == 0:
            self.MaxWindSpeed_List = [float(50)]

    try:
        self.ASTtol_value_from = float(input('Enter the ASTtol value from (if omitted, will be 0.1): '))
    except ValueError:
        self.ASTtol_value_from = float(0.1)
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        try:
            self.ASTtol_value_from = float(input('     Enter the ASTtol value from (if omitted, will be 0.1): '))
        except ValueError:
            self.ASTtol_value_from = float(0.1)

    try:
        self.ASTtol_value_to_input = float(input('Enter the ASTtol value to (if omitted, will be 0.1): '))
    except ValueError:
        self.ASTtol_value_to_input = float(0.1)
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        try:
            self.ASTtol_value_to_input = float(input('     Enter the ASTtol value to (if omitted, will be 0.1): '))
        except ValueError:
            self.ASTtol_value_to_input = float(0.1)

    try:
        self.ASTtol_value_steps = float(input('Enter the ASTtol value steps (if omitted, will be 0.1): '))
    except ValueError:
        self.ASTtol_value_steps = float(0.1)
    while input('          Are you sure the numbers are correct? [y or [] / n]: ') == 'n':
        try:
            self.ASTtol_value_steps = float(input('     Enter the ASTtol value steps (if omitted, will be 0.1): '))
        except ValueError:
            self.ASTtol_value_steps = float(0.1)


def genIDFSingleZone(self,
         AdapStand=None,
         CAT=None,
         ComfMod=None,
         ASTtol_start=0.1,
         ASTtol_end_input=0.1,
         ASTtol_steps=0.1,
         NameSuffix='',
         verboseMode: bool =True,
         confirmGen: bool =None
         ):
    """Generate IDFs in SingleZone."""
    import os
    from os import listdir
    import numpy
    from eppy import modeleditor
    from eppy.modeleditor import IDF

    arguments = (AdapStand is None,
                 CAT is None,
                 ComfMod is None,
                 ASTtol_start == 0.1,
                 ASTtol_end_input == 0.1,
                 ASTtol_steps == 0.1)
    # todo if the user adds some arguments but leave others by default, might get some error; so raise error needs to be added
    if all(arguments):
        self.ASTtol_value_to = self.ASTtol_value_to_input+self.ASTtol_value_steps
    else:
        ASTtol_end = ASTtol_end_input + ASTtol_steps

    if all(arguments):
        self.ASTtol_value_from = round(self.ASTtol_value_from, 2)
        self.ASTtol_value_to = round(self.ASTtol_value_to, 2)
        self.ASTtol_value_steps = round(self.ASTtol_value_steps, 2)
        pass
    else:
        self.AdapStand_List = AdapStand
        self.CAT_List = CAT
        self.ComfMod_List = ComfMod
        self.ASTtol_value_from = round(ASTtol_start, 2)
        self.ASTtol_value_to = round(ASTtol_end, 2)
        self.ASTtol_value_steps = round(ASTtol_steps, 2)

    if NameSuffix == '':
        suffix = ''
    else:
        suffix = '[' + NameSuffix

    filelist_pymod = ([file for file in listdir() if file.endswith('_pymod.idf')])
    filelist_pymod = ([file.split('.idf')[0] for file in filelist_pymod])
    # print(filelist_pymod)

    outputlist = []
    for file in filelist_pymod:
        filename = file
        for AdapStand_value in self.AdapStand_List:
            if AdapStand_value == 0:
                for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                    outputname = (
                            filename
                            + '[AS_CTE'
                            + '[CA_X'
                            + '[CM_X'
                            + '[AT_' + repr(ASTtol_value)
                            + suffix
                            + '.idf'
                    )
                    outputlist.append(outputname)
            elif AdapStand_value == 1:
                for CAT_value in self.CAT_List:
                    if CAT_value not in range(0, 4):
                        continue
                    else:
                        for ComfMod_value in self.ComfMod_List:
                            for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                outputname = (
                                    filename
                                    + '[AS_EN16798'
                                    + '[CA_'+repr(CAT_value)
                                    + '[CM_'+repr(ComfMod_value)
                                    + '[AT_'+repr(ASTtol_value)
                                    + suffix
                                    + '.idf'
                                    )
                                outputlist.append(outputname)
            elif AdapStand_value == 2:
                for CAT_value in self.CAT_List:
                    if CAT_value not in range(80, 91, 10):
                        continue
                    else:
                        for ComfMod_value in self.ComfMod_List:
                            for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                outputname = (
                                    filename
                                    + '[AS_ASHRAE55'
                                    + '[CA_'+repr(CAT_value)
                                    + '[CM_'+repr(ComfMod_value)
                                    + '[AT_'+repr(ASTtol_value)
                                    + suffix
                                    + '.idf'
                                    )
                                outputlist.append(outputname)

    if verboseMode:
        print('The list of output IDFs is going to be:')
        print(*outputlist, sep="\n")
        print(f'And the total number of output IDFs is going to be {len(outputlist)}')

    if confirmGen is None:
        confirmGen = input('Do you still want to run ACCIS? [y/n]: ')
        if confirmGen == 'y':
            confirmGen = True
        else:
            confirmGen = False

    if confirmGen == True:
        for file in filelist_pymod:
            filename = file

            fname1 = filename+'.idf'
            if verboseMode:
                print(fname1)
            idf1 = IDF(fname1)

            # print(filename)
            SetInputData = ([program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetInputData'])
            # Making IDFs: the output file path can not contain the following characters: & ^ , = % " \ / : * ? " < > |
            for AdapStand_value in self.AdapStand_List:
                SetInputData[0].Program_Line_1 = 'set AdapStand  =  '+repr(AdapStand_value)
                if AdapStand_value == 0:
                    SetInputData[0].Program_Line_2 = 'set CAT = 1'
                    SetInputData[0].Program_Line_3 = 'set ComfMod = 0'
                    for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                        SetInputData[0].Program_Line_4 = 'set ACSTtol = '+repr(-ASTtol_value)
                        SetInputData[0].Program_Line_5 = 'set AHSTtol = '+repr(ASTtol_value)
                        outputname = (
                            filename
                            + '[AS_CTE'
                            + '[CA_X'
                            + '[CM_X'
                            + '[AT_' + repr(ASTtol_value)
                            + suffix
                            + '.idf'
                        )
                        if verboseMode:
                            print(outputname)
                        idf1.savecopy(outputname)
                elif AdapStand_value == 1:
                    for CAT_value in self.CAT_List:
                        if CAT_value not in range(0, 4):
                            continue
                        else:
                            SetInputData[0].Program_Line_2 = 'set CAT = '+repr(CAT_value)
                            for ComfMod_value in self.ComfMod_List:
                                SetInputData[0].Program_Line_3 = 'set ComfMod = '+repr(ComfMod_value)
                                for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                    SetInputData[0].Program_Line_4 = 'set ACSTtol = '+repr(-ASTtol_value)
                                    SetInputData[0].Program_Line_5 = 'set AHSTtol = '+repr(ASTtol_value)
                                    outputname = (
                                        filename
                                        + '[AS_EN16798'
                                        + '[CA_'+repr(CAT_value)
                                        + '[CM_'+repr(ComfMod_value)
                                        + '[AT_'+repr(ASTtol_value)
                                        + suffix
                                        + '.idf'
                                        )
                                    if verboseMode:
                                        print(outputname)
                                    idf1.savecopy(outputname)
                elif AdapStand_value == 2:
                    for CAT_value in self.CAT_List:
                        if CAT_value not in range(80, 91, 10):
                            continue
                        else:
                            SetInputData[0].Program_Line_2 = 'set CAT = '+repr(CAT_value)
                            for ComfMod_value in self.ComfMod_List:
                                SetInputData[0].Program_Line_3 = 'set ComfMod = '+repr(ComfMod_value)
                                for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                    SetInputData[0].Program_Line_4 = 'set ACSTtol = '+repr(-ASTtol_value)
                                    SetInputData[0].Program_Line_5 = 'set AHSTtol = '+repr(ASTtol_value)
                                    outputname = (
                                        filename
                                        + '[AS_ASHRAE55'
                                        + '[CA_'+repr(CAT_value)
                                        + '[CM_'+repr(ComfMod_value)
                                        + '[AT_'+repr(ASTtol_value)
                                        + suffix
                                        + '.idf'
                                        )
                                    if verboseMode:
                                        print(outputname)
                                    idf1.savecopy(outputname)
    elif confirmGen == False:
        if verboseMode:
            print('IDF generation has been shut down')

    filelist_pymod = ([file for file in listdir() if file.endswith('_pymod.idf')])
    for file in filelist_pymod:
        os.remove(file)

    # del SetInputData


def genIDFMultipleZone(self,
                       AdapStand=None,
                       CAT=None,
                       ComfMod=None,
                       HVACmode=None,
                       VentCtrl=None,
                       VSToffset=[0],
                       MinOToffset=[50],
                       MaxWindSpeed=[50],
                       ASTtol_start=0.1,
                       ASTtol_end_input=0.1,
                       ASTtol_steps=0.1,
                       NameSuffix='',
                       verboseMode: bool = True,
                       confirmGen: bool = None
                       ):
    """Generate IDFs in MultipleZone."""
    import os
    from os import listdir
    import numpy
    from eppy import modeleditor
    from eppy.modeleditor import IDF
    import time
    from tqdm import tqdm
    
    arguments = (AdapStand is None,
                 CAT is None,
                 ComfMod is None,
                 HVACmode is None,
                 VentCtrl is None,
                 VSToffset == [0],
                 MinOToffset == [50],
                 MaxWindSpeed == [50],
                 ASTtol_start == 0.1,
                 ASTtol_end_input == 0.1,
                 ASTtol_steps == 0.1)
    # todo if the user adds some arguments but leave others by default, might get some error; so raise error needs to be added
    if all(arguments):
        self.ASTtol_value_to = self.ASTtol_value_to_input+self.ASTtol_value_steps
    else:
        ASTtol_end = ASTtol_end_input + ASTtol_steps

    if all(arguments):
        self.ASTtol_value_from = round(self.ASTtol_value_from, 2)
        self.ASTtol_value_to = round(self.ASTtol_value_to, 2)
        self.ASTtol_value_steps = round(self.ASTtol_value_steps, 2)
    else:
        self.AdapStand_List = AdapStand
        self.CAT_List = CAT
        self.ComfMod_List = ComfMod
        self.HVACmode_List = HVACmode
        self.VentCtrl_List = VentCtrl
        self.VSToffset_List = VSToffset
        self.MinOToffset_List = MinOToffset
        self.MaxWindSpeed_List = MaxWindSpeed
        self.ASTtol_value_from = round(ASTtol_start, 2)
        self.ASTtol_value_to = round(ASTtol_end, 2)
        self.ASTtol_value_steps = round(ASTtol_steps, 2)

    if NameSuffix == '':
        suffix = ''
    else:
        suffix = '[' + NameSuffix

    filelist_pymod = ([file for file in listdir() if file.endswith('_pymod.idf')])
    filelist_pymod = ([file.split('.idf')[0] for file in filelist_pymod])
    # print(filelist_pymod)

    outputlist = []
    for file in filelist_pymod:
        filename = file
        for AdapStand_value in self.AdapStand_List:
            if AdapStand_value == 0:
                for HVACmode_value in self.HVACmode_List:
                    if HVACmode_value == 0:
                        for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                            outputname = (
                                filename
                                + '[AS_CTE'
                                + '[CA_X'
                                + '[CM_X'
                                + '[HM_' + repr(HVACmode_value)
                                + '[VC_X'
                                + '[VO_X'
                                + '[MT_X'
                                + '[MW_X'
                                + '[AT_' + repr(ASTtol_value)
                                + suffix
                                + '.idf'
                                )
                            outputlist.append(outputname)
                    else:
                        for VentCtrl_value in self.VentCtrl_List:
                            for VSToffset_value in self.VSToffset_List:
                                for MinOToffset_value in self.MinOToffset_List:
                                    for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                        for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                            outputname = (
                                                filename
                                                + '[AS_CTE'
                                                + '[CA_X'
                                                + '[CM_X'
                                                + '[HM_' + repr(HVACmode_value)
                                                + '[VC_' + repr(VentCtrl_value)
                                                + '[VO_' + repr(VSToffset_value)
                                                + '[MT_' + repr(MinOToffset_value)
                                                + '[MW_' + repr(MaxWindSpeed_value)
                                                + '[AT_' + repr(ASTtol_value)
                                                + suffix
                                                + '.idf'
                                                )
                                            outputlist.append(outputname)
            elif AdapStand_value == 1:
                for CAT_value in self.CAT_List:
                    if CAT_value not in range(0, 4):
                        continue
                    else:
                        for ComfMod_value in self.ComfMod_List:
                            for HVACmode_value in self.HVACmode_List:
                                if HVACmode_value == 0:
                                    for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                        outputname = (
                                            filename
                                            + '[AS_EN16798'
                                            + '[CA_' + repr(CAT_value)
                                            + '[CM_' + repr(ComfMod_value)
                                            + '[HM_' + repr(HVACmode_value)
                                            + '[VC_X'
                                            + '[VO_X'
                                            + '[MT_X'
                                            + '[MW_X'
                                            + '[AT_' + repr(ASTtol_value)
                                            + suffix
                                            + '.idf'
                                            )
                                        outputlist.append(outputname)
                                else:
                                    for VentCtrl_value in self.VentCtrl_List:
                                        for VSToffset_value in self.VSToffset_List:
                                            for MinOToffset_value in self.MinOToffset_List:
                                                for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                                    for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                                        outputname = (
                                                            filename
                                                            + '[AS_EN16798'
                                                            + '[CA_' + repr(CAT_value)
                                                            + '[CM_' + repr(ComfMod_value)
                                                            + '[HM_' + repr(HVACmode_value)
                                                            + '[VC_' + repr(VentCtrl_value)
                                                            + '[VO_' + repr(VSToffset_value)
                                                            + '[MT_' + repr(MinOToffset_value)
                                                            + '[MW_' + repr(MaxWindSpeed_value)
                                                            + '[AT_' + repr(ASTtol_value)
                                                            + suffix
                                                            + '.idf'
                                                            )
                                                        outputlist.append(outputname)
            elif AdapStand_value == 2:
                for CAT_value in self.CAT_List:
                    if CAT_value not in range(80, 91, 10):
                        continue
                    else:
                        for ComfMod_value in self.ComfMod_List:
                            for HVACmode_value in self.HVACmode_List:
                                if HVACmode_value == 0:
                                    for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                        outputname = (
                                            filename
                                            + '[AS_EN16798'
                                            + '[CA_' + repr(CAT_value)
                                            + '[CM_' + repr(ComfMod_value)
                                            + '[HM_' + repr(HVACmode_value)
                                            + '[VC_X'
                                            + '[VO_X'
                                            + '[MT_X'
                                            + '[MW_X'
                                            + '[AT_' + repr(ASTtol_value)
                                            + suffix
                                            + '.idf'
                                            )
                                        outputlist.append(outputname)
                                else:
                                    for VentCtrl_value in self.VentCtrl_List:
                                        for VSToffset_value in self.VSToffset_List:
                                            for MinOToffset_value in self.MinOToffset_List:
                                                for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                                    for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                                        outputname = (
                                                            filename
                                                            + '[AS_ASHRAE55'
                                                            + '[CA_' + repr(CAT_value)
                                                            + '[CM_' + repr(ComfMod_value)
                                                            + '[HM_' + repr(HVACmode_value)
                                                            + '[VC_' + repr(VentCtrl_value)
                                                            + '[VO_' + repr(VSToffset_value)
                                                            + '[MT_' + repr(MinOToffset_value)
                                                            + '[MW_' + repr(MaxWindSpeed_value)
                                                            + '[AT_' + repr(ASTtol_value)
                                                            + suffix
                                                            + '.idf'
                                                            )
                                                        outputlist.append(outputname)
    if verboseMode:
        print('The list of output IDFs is going to be:')
        print(*outputlist, sep="\n")
        print(f'And the total number of output IDFs is going to be {len(outputlist)}')

    if confirmGen is None:
        confirmGen = input('Do you still want to run ACCIS? [y/n]: ')
        if confirmGen == 'y':
            confirmGen = True
        else:
            confirmGen = False

    if confirmGen == True:
        # pbar = tqdm(total=len(outputlist))
        for file in filelist_pymod:
            filename = file

            fname1 = filename+'.idf'
            if verboseMode:
                print(fname1)
            idf1 = IDF(fname1)

            # print(filename)

            SetInputData = ([program for program in idf1.idfobjects['EnergyManagementSystem:Program'] if program.Name == 'SetInputData'])

            for AdapStand_value in self.AdapStand_List:
                SetInputData[0].Program_Line_1 = 'set AdapStand = '+repr(AdapStand_value)
                if AdapStand_value == 0:
                    SetInputData[0].Program_Line_2 = 'set CAT = 1'
                    SetInputData[0].Program_Line_3 = 'set ComfMod = 0'
                    for HVACmode_value in self.HVACmode_List:
                        SetInputData[0].Program_Line_4 = 'set HVACmode = '+repr(HVACmode_value)
                        if HVACmode_value == 0:
                            for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                SetInputData[0].Program_Line_5 = 'set ACSTtol = '+repr(-ASTtol_value)
                                SetInputData[0].Program_Line_6 = 'set AHSTtol = '+repr(ASTtol_value)
                                outputname = (
                                    filename
                                    + '[AS_CTE'
                                    + '[CA_X'
                                    + '[CM_X'
                                    + '[HM_' + repr(HVACmode_value)
                                    + '[VC_X'
                                    + '[VO_X'
                                    + '[MT_X'
                                    + '[MW_X'
                                    + '[AT_' + repr(ASTtol_value)
                                    + suffix
                                    + '.idf'
                                    )
                                if verboseMode:
                                    print(outputname)
                                    # time.sleep(0.1)
                                    # pbar.update(1)
                                idf1.savecopy(outputname)
                        else:
                            for VentCtrl_value in self.VentCtrl_List:
                                SetInputData[0].Program_Line_5 = 'set VentCtrl = '+repr(VentCtrl_value)
                                for VSToffset_value in self.VSToffset_List:
                                    SetInputData[0].Program_Line_6 = 'set VSToffset = '+repr(VSToffset_value)
                                    for MinOToffset_value in self.MinOToffset_List:
                                        SetInputData[0].Program_Line_7 = 'set MinOToffset = '+repr(MinOToffset_value)
                                        for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                            SetInputData[0].Program_Line_8 = 'set MaxWindSpeed = '+repr(MaxWindSpeed_value)
                                            for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                                SetInputData[0].Program_Line_9 = 'set ACSTtol = '+repr(-ASTtol_value)
                                                SetInputData[0].Program_Line_10 = 'set AHSTtol = '+repr(ASTtol_value)
                                                outputname = (
                                                    filename
                                                    + '[AS_CTE'
                                                    + '[CA_X'
                                                    + '[CM_X'
                                                    + '[HM_' + repr(HVACmode_value)
                                                    + '[VC_' + repr(VentCtrl_value)
                                                    + '[VO_' + repr(VSToffset_value)
                                                    + '[MT_' + repr(MinOToffset_value)
                                                    + '[MW_' + repr(MaxWindSpeed_value)
                                                    + '[AT_' + repr(ASTtol_value)
                                                    + suffix
                                                    + '.idf'
                                                    )
                                                if verboseMode:
                                                    print(outputname)
                                                    # time.sleep(0.1)
                                                    # pbar.update(1)
                                                idf1.savecopy(outputname)
                elif AdapStand_value == 1:
                    for CAT_value in self.CAT_List:
                        if CAT_value not in range(0, 4):
                            continue
                        else:
                            SetInputData[0].Program_Line_2 = 'set CAT = '+repr(CAT_value)
                            for ComfMod_value in self.ComfMod_List:
                                SetInputData[0].Program_Line_3 = 'set ComfMod = '+repr(ComfMod_value)
                                for HVACmode_value in self.HVACmode_List:
                                    SetInputData[0].Program_Line_4 = 'set HVACmode = '+repr(HVACmode_value)
                                    if HVACmode_value == 0:
                                        for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                            SetInputData[0].Program_Line_9 = 'set ACSTtol = '+repr(-ASTtol_value)
                                            SetInputData[0].Program_Line_10 = 'set AHSTtol = '+repr(ASTtol_value)
                                            outputname = (
                                                filename
                                                + '[AS_EN16798'
                                                + '[CA_' + repr(CAT_value)
                                                + '[CM_' + repr(ComfMod_value)
                                                + '[HM_' + repr(HVACmode_value)
                                                + '[VC_X'
                                                + '[VO_X'
                                                + '[MT_X'
                                                + '[MW_X'
                                                + '[AT_' + repr(ASTtol_value)
                                                + suffix
                                                + '.idf'
                                                )
                                            if verboseMode:
                                                print(outputname)
                                                # time.sleep(0.1)
                                                # pbar.update(1)
                                            idf1.savecopy(outputname)
                                    else:
                                        for VentCtrl_value in self.VentCtrl_List:
                                            SetInputData[0].Program_Line_5 = 'set VentCtrl = '+repr(VentCtrl_value)
                                            for VSToffset_value in self.VSToffset_List:
                                                SetInputData[0].Program_Line_6 = 'set VSToffset = '+repr(VSToffset_value)
                                                for MinOToffset_value in self.MinOToffset_List:
                                                    SetInputData[0].Program_Line_7 = 'set MinOToffset = '+repr(MinOToffset_value)
                                                    for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                                        SetInputData[0].Program_Line_8 = 'set MaxWindSpeed = '+repr(MaxWindSpeed_value)
                                                        for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                                            SetInputData[0].Program_Line_9 = 'set ACSTtol = '+repr(-ASTtol_value)
                                                            SetInputData[0].Program_Line_10 = 'set AHSTtol = '+repr(ASTtol_value)
                                                            outputname = (
                                                                filename
                                                                + '[AS_EN16798'
                                                                + '[CA_' + repr(CAT_value)
                                                                + '[CM_' + repr(ComfMod_value)
                                                                + '[HM_' + repr(HVACmode_value)
                                                                + '[VC_' + repr(VentCtrl_value)
                                                                + '[VO_' + repr(VSToffset_value)
                                                                + '[MT_' + repr(MinOToffset_value)
                                                                + '[MW_' + repr(MaxWindSpeed_value)
                                                                + '[AT_' + repr(ASTtol_value)
                                                                + suffix
                                                                + '.idf'
                                                                )
                                                            if verboseMode:
                                                                print(outputname)
                                                                # time.sleep(0.1)
                                                                # pbar.update(1)
                                                            idf1.savecopy(outputname)
                elif AdapStand_value == 2:
                    for CAT_value in self.CAT_List:
                        if CAT_value not in range(80, 91, 10):
                            continue
                        else:
                            SetInputData[0].Program_Line_2 = 'set CAT = '+repr(CAT_value)
                            for ComfMod_value in self.ComfMod_List:
                                SetInputData[0].Program_Line_3 = 'set ComfMod = '+repr(ComfMod_value)
                                for HVACmode_value in self.HVACmode_List:
                                    SetInputData[0].Program_Line_4 = 'set HVACmode = '+repr(HVACmode_value)
                                    if HVACmode_value == 0:
                                        for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                            SetInputData[0].Program_Line_9 = 'set ACSTtol = '+repr(-ASTtol_value)
                                            SetInputData[0].Program_Line_10 = 'set AHSTtol = '+repr(ASTtol_value)
                                            outputname = (
                                                filename
                                                + '[AS_EN16798'
                                                + '[CA_' + repr(CAT_value)
                                                + '[CM_' + repr(ComfMod_value)
                                                + '[HM_' + repr(HVACmode_value)
                                                + '[VC_X'
                                                + '[VO_X'
                                                + '[MT_X'
                                                + '[MW_X'
                                                + '[AT_' + repr(ASTtol_value)
                                                + suffix
                                                + '.idf'
                                                )
                                            if verboseMode:
                                                print(outputname)
                                                # time.sleep(0.1)
                                                # pbar.update(1)
                                            idf1.savecopy(outputname)
                                    else:
                                        for VentCtrl_value in self.VentCtrl_List:
                                            SetInputData[0].Program_Line_5 = 'set VentCtrl = '+repr(VentCtrl_value)
                                            for VSToffset_value in self.VSToffset_List:
                                                SetInputData[0].Program_Line_6 = 'set VSToffset = '+repr(VSToffset_value)
                                                for MinOToffset_value in self.MinOToffset_List:
                                                    SetInputData[0].Program_Line_7 = 'set MinOToffset = '+repr(MinOToffset_value)
                                                    for MaxWindSpeed_value in self.MaxWindSpeed_List:
                                                        SetInputData[0].Program_Line_8 = 'set MaxWindSpeed = '+repr(MaxWindSpeed_value)
                                                        for ASTtol_value in numpy.arange(self.ASTtol_value_from, self.ASTtol_value_to, self.ASTtol_value_steps):
                                                            SetInputData[0].Program_Line_9 = 'set ACSTtol = '+repr(-ASTtol_value)
                                                            SetInputData[0].Program_Line_10 = 'set AHSTtol = '+repr(ASTtol_value)
                                                            outputname = (
                                                                filename
                                                                + '[AS_ASHRAE55'
                                                                + '[CA_' + repr(CAT_value)
                                                                + '[CM_' + repr(ComfMod_value)
                                                                + '[HM_' + repr(HVACmode_value)
                                                                + '[VC_' + repr(VentCtrl_value)
                                                                + '[VO_' + repr(VSToffset_value)
                                                                + '[MT_' + repr(MinOToffset_value)
                                                                + '[MW_' + repr(MaxWindSpeed_value)
                                                                + '[AT_' + repr(ASTtol_value)
                                                                + suffix
                                                                + '.idf'
                                                                )
                                                            if verboseMode:
                                                                print(outputname)
                                                                # time.sleep(0.1)
                                                                # pbar.update(1)
                                                            idf1.savecopy(outputname)
        # pbar.close()
    elif confirmGen == False:
        if verboseMode:
            print('IDF generation has been shut down')

    filelist_pymod = ([file for file in listdir() if file.endswith('_pymod.idf')])
    for file in filelist_pymod:
        os.remove(file)

    # del SetInputData



# -*- coding: utf-8 -*-
"""
Created on Sun Feb 28 17:03:48 2021

@author: Daniel
"""

import pytest
from accim.sim import accim_Main

@pytest.fixture()
def IDFobject():
    from eppy.modeleditor import IDF

    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)

    z = accim_Main.accimInstance(
        filename_temp='TestModel_SingleZone',
        ScriptType='sz',
        EnergyPlus_version='ep94',
        verboseMode=False)
    return z

def test_addEMSProgramsBase(IDFobject):

    from eppy.modeleditor import IDF

    IDFobject.addEMSProgramsBase(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)

    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    programlist = ([program.Name for program in idf1.idfobjects['EnergyManagementSystem:Program']])
    assert ('SetComfTemp' in programlist)

    SetComfTemp = ([program
                    for program
                    in idf1.idfobjects['EnergyManagementSystem:Program']
                    if program.Name == 'SetComfTemp'])
    assert SetComfTemp[0].Program_Line_1 == 'if AdapStand == 1'
    assert SetComfTemp[0].Program_Line_2 == 'set ComfTemp = RMOT*0.33+18.8'
    assert SetComfTemp[0].Program_Line_3 == 'elseif AdapStand == 2'
    assert SetComfTemp[0].Program_Line_4 == 'set ComfTemp = PMOT*0.31+17.8'
    assert SetComfTemp[0].Program_Line_5 == 'endif'

    zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in idf1.idfobjects['ZONE']])])

    for zonename in zonenames:
        CountHours = ([program
                        for program
                        in idf1.idfobjects['EnergyManagementSystem:Program']
                        if program.Name == 'CountHours_' + zonename])
        assert CountHours[0].Name == 'CountHours_' + zonename
        assert CountHours[0].Program_Line_1 == 'if (AdapStand == 1 )'
        assert CountHours[0].Program_Line_2 == 'if (RMOT >= AHSTall) && (RMOT <= ACSTaul)'
        assert CountHours[0].Program_Line_3 == 'if (' + zonename + '_OpT <= ACSTnoTol)'
        assert CountHours[0].Program_Line_4 == 'if (' + zonename + '_OpT >= AHSTnoTol)'
        assert CountHours[0].Program_Line_5 == 'set ComfHours_' + zonename + ' = 1*ZoneTimeStep'
        assert CountHours[0].Program_Line_6 == 'set DiscomfAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_7 == 'set DiscomfAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_8 == 'set DiscomfNonAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_9 == 'set DiscomfNonAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_10 == 'endif'
        assert CountHours[0].Program_Line_11 == 'elseif (' + zonename + '_OpT > ACSTnoTol)'
        assert CountHours[0].Program_Line_12 == 'set ComfHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_13 == 'set DiscomfAppHotHours_' + zonename + ' = 1*ZoneTimeStep'
        assert CountHours[0].Program_Line_14 == 'set DiscomfAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_15 == 'set DiscomfNonAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_16 == 'set DiscomfNonAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_17 == 'elseif (' + zonename + '_OpT < AHSTnoTol)'
        assert CountHours[0].Program_Line_18 == 'set ComfHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_19 == 'set DiscomfAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_20 == 'set DiscomfAppColdHours_' + zonename + ' = 1*ZoneTimeStep'
        assert CountHours[0].Program_Line_21 == 'set DiscomfNonAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_22 == 'set DiscomfNonAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_23 == 'endif'
        assert CountHours[0].Program_Line_24 == 'elseif (RMOT > ACSTaul)'
        assert CountHours[0].Program_Line_25 == 'set ComfHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_26 == 'set DiscomfAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_27 == 'set DiscomfAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_28 == 'set DiscomfNonAppHotHours_' + zonename + ' = 1*ZoneTimeStep'
        assert CountHours[0].Program_Line_29 == 'set DiscomfNonAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_30 == 'elseif (RMOT < AHSTall)'
        assert CountHours[0].Program_Line_31 == 'set ComfHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_32 == 'set DiscomfAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_33 == 'set DiscomfAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_34 == 'set DiscomfNonAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_35 == 'set DiscomfNonAppColdHours_' + zonename + ' = 1*ZoneTimeStep'
        assert CountHours[0].Program_Line_36 == 'endif'
        assert CountHours[0].Program_Line_37 == 'elseif (AdapStand == 2 )'
        assert CountHours[0].Program_Line_38 == 'if (PMOT >= AHSTall) && (PMOT <= ACSTaul)'
        assert CountHours[0].Program_Line_39 == 'if (' + zonename + '_OpT <= ACSTnoTol)'
        assert CountHours[0].Program_Line_40 == 'if (' + zonename + '_OpT >= AHSTnoTol)'
        assert CountHours[0].Program_Line_41 == 'set ComfHours_' + zonename + ' = 1*ZoneTimeStep'
        assert CountHours[0].Program_Line_42 == 'set DiscomfAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_43 == 'set DiscomfAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_44 == 'set DiscomfNonAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_45 == 'set DiscomfNonAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_46 == 'endif'
        assert CountHours[0].Program_Line_47 == 'elseif (' + zonename + '_OpT > ACSTnoTol)'
        assert CountHours[0].Program_Line_48 == 'set ComfHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_49 == 'set DiscomfAppHotHours_' + zonename + ' = 1*ZoneTimeStep'
        assert CountHours[0].Program_Line_50 == 'set DiscomfAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_51 == 'set DiscomfNonAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_52 == 'set DiscomfNonAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_53 == 'elseif (' + zonename + '_OpT < AHSTnoTol)'
        assert CountHours[0].Program_Line_54 == 'set ComfHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_55 == 'set DiscomfAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_56 == 'set DiscomfAppColdHours_' + zonename + ' = 1*ZoneTimeStep'
        assert CountHours[0].Program_Line_57 == 'set DiscomfNonAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_58 == 'set DiscomfNonAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_59 == 'endif'
        assert CountHours[0].Program_Line_60 == 'elseif (PMOT > ACSTaul)'
        assert CountHours[0].Program_Line_61 == 'set ComfHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_62 == 'set DiscomfAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_63 == 'set DiscomfAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_64 == 'set DiscomfNonAppHotHours_' + zonename + ' = 1*ZoneTimeStep'
        assert CountHours[0].Program_Line_65 == 'set DiscomfNonAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_66 == 'elseif (PMOT < AHSTall)'
        assert CountHours[0].Program_Line_67 == 'set ComfHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_68 == 'set DiscomfAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_69 == 'set DiscomfAppColdHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_70 == 'set DiscomfNonAppHotHours_' + zonename + ' = 0'
        assert CountHours[0].Program_Line_71 == 'set DiscomfNonAppColdHours_' + zonename + ' = 1*ZoneTimeStep'
        assert CountHours[0].Program_Line_72 == 'endif'
        assert CountHours[0].Program_Line_73 == 'endif'

    SetAppLimits = ([program
                    for program
                    in idf1.idfobjects['EnergyManagementSystem:Program']
                    if program.Name == 'SetAppLimits'])
    assert SetAppLimits[0].Name == 'SetAppLimits'
    assert SetAppLimits[0].Program_Line_1 == 'If AdapStand == 1'
    assert SetAppLimits[0].Program_Line_2 == 'set ACSTaul = 30'
    assert SetAppLimits[0].Program_Line_3 == 'set ACSTall = 10'
    assert SetAppLimits[0].Program_Line_4 == 'set AHSTaul = 30'
    assert SetAppLimits[0].Program_Line_5 == 'set AHSTall = 10'
    assert SetAppLimits[0].Program_Line_6 == 'elseif AdapStand == 2'
    assert SetAppLimits[0].Program_Line_7 == 'set ACSTaul = 33.5'
    assert SetAppLimits[0].Program_Line_8 == 'set ACSTall = 10'
    assert SetAppLimits[0].Program_Line_9 == 'set AHSTaul = 33.5'
    assert SetAppLimits[0].Program_Line_10 == 'set AHSTall = 10'
    assert SetAppLimits[0].Program_Line_11 == 'else'
    assert SetAppLimits[0].Program_Line_12 == 'set ACSTaul = 50'
    assert SetAppLimits[0].Program_Line_13 == 'set ACSTall = 50'
    assert SetAppLimits[0].Program_Line_14 == 'set AHSTaul = 50'
    assert SetAppLimits[0].Program_Line_15 == 'set AHSTall = 50'
    assert SetAppLimits[0].Program_Line_16 == 'endif'

    
    ApplyCAT = ([program
                 for program
                 in idf1.idfobjects['EnergyManagementSystem:Program']
                 if program.Name == 'ApplyCAT'])
    assert ApplyCAT[0].Name == 'ApplyCAT'
    assert ApplyCAT[0].Program_Line_1 == 'if (AdapStand == 1 )'
    assert ApplyCAT[0].Program_Line_2 == 'if (CAT == 1)'
    assert ApplyCAT[0].Program_Line_3 == 'set ACSToffset = 2'
    assert ApplyCAT[0].Program_Line_4 == 'set AHSToffset = -3'
    assert ApplyCAT[0].Program_Line_5 == 'elseif (CAT == 2)'
    assert ApplyCAT[0].Program_Line_6 == 'set ACSToffset = 3'
    assert ApplyCAT[0].Program_Line_7 == 'set AHSToffset = -4'
    assert ApplyCAT[0].Program_Line_8 == 'elseif (CAT == 3)'
    assert ApplyCAT[0].Program_Line_9 == 'set ACSToffset = 4'
    assert ApplyCAT[0].Program_Line_10 == 'set AHSToffset = -5'
    assert ApplyCAT[0].Program_Line_11 == 'endif'
    assert ApplyCAT[0].Program_Line_12 == 'elseif (AdapStand == 2 )'
    assert ApplyCAT[0].Program_Line_13 == 'if (CAT == 90)'
    assert ApplyCAT[0].Program_Line_14 == 'set ACSToffset = 2.5'
    assert ApplyCAT[0].Program_Line_15 == 'set AHSToffset = -2.5'
    assert ApplyCAT[0].Program_Line_16 == 'elseif (CAT == 80)'
    assert ApplyCAT[0].Program_Line_17 == 'set ACSToffset = 3.5'
    assert ApplyCAT[0].Program_Line_18 == 'set AHSToffset = -3.5'
    assert ApplyCAT[0].Program_Line_19 == 'endif'
    assert ApplyCAT[0].Program_Line_20 == 'endif'
    
    SetAST = ([program
               for program
               in idf1.idfobjects['EnergyManagementSystem:Program']
               if program.Name == 'SetAST'])
    assert SetAST[0].Name == 'SetAST'
    assert SetAST[0].Program_Line_1 == 'if (AdapStand == 0) && (CurrentTime < 7)'
    assert SetAST[0].Program_Line_2 == 'set ACST = 27+ACSTtol'
    assert SetAST[0].Program_Line_3 == 'set AHST = 17+AHSTtol'
    assert SetAST[0].Program_Line_4 == 'elseif (AdapStand == 0) && (CurrentTime < 15)'
    assert SetAST[0].Program_Line_5 == 'set ACST = 25'
    assert SetAST[0].Program_Line_6 == 'set AHST = 20+AHSTtol'
    assert SetAST[0].Program_Line_7 == 'elseif (AdapStand == 0) && (CurrentTime < 23)'
    assert SetAST[0].Program_Line_8 == 'set ACST = 25+ACSTtol'
    assert SetAST[0].Program_Line_9 == 'set AHST = 20+AHSTtol'
    assert SetAST[0].Program_Line_10 == 'elseif (AdapStand == 0) && (CurrentTime < 24)'
    assert SetAST[0].Program_Line_11 == 'set ACST = 27+ACSTtol'
    assert SetAST[0].Program_Line_12 == 'set AHST = 17+AHSTtol'
    assert SetAST[0].Program_Line_13 == 'endif'
    assert SetAST[0].Program_Line_14 == 'if (AdapStand == 1) && (ComfMod == 0)'
    assert SetAST[0].Program_Line_15 == 'if (DayOfYear >= 121) && (DayOfYear < 274)'
    assert SetAST[0].Program_Line_16 == 'if (CAT==1)'
    assert SetAST[0].Program_Line_17 == 'set ACST = 25.5+ACSTtol'
    assert SetAST[0].Program_Line_18 == 'elseif (CAT==2)'
    assert SetAST[0].Program_Line_19 == 'set ACST = 26+ACSTtol'
    assert SetAST[0].Program_Line_20 == 'elseif (CAT==3)'
    assert SetAST[0].Program_Line_21 == 'set ACST = 27+ACSTtol'
    assert SetAST[0].Program_Line_22 == 'endif'
    assert SetAST[0].Program_Line_23 == 'else'
    assert SetAST[0].Program_Line_24 == 'if (CAT==1)'
    assert SetAST[0].Program_Line_25 == 'set ACST = 25+ACSTtol'
    assert SetAST[0].Program_Line_26 == 'elseif (CAT==2)'
    assert SetAST[0].Program_Line_27 == 'set ACST = 25+ACSTtol'
    assert SetAST[0].Program_Line_28 == 'elseif (CAT==3)'
    assert SetAST[0].Program_Line_29 == 'set ACST = 25+ACSTtol'
    assert SetAST[0].Program_Line_30 == 'endif'
    assert SetAST[0].Program_Line_31 == 'endif'
    assert SetAST[0].Program_Line_32 == 'endif'
    assert SetAST[0].Program_Line_33 == 'if (AdapStand == 1) && (ComfMod == 0)'
    assert SetAST[0].Program_Line_34 == 'if (DayOfYear >= 121) && (DayOfYear < 274)'
    assert SetAST[0].Program_Line_35 == 'if (CAT==1)'
    assert SetAST[0].Program_Line_36 == 'set AHST = 23.5+AHSTtol'
    assert SetAST[0].Program_Line_37 == 'elseif (CAT==2)'
    assert SetAST[0].Program_Line_38 == 'set AHST = 23+AHSTtol'
    assert SetAST[0].Program_Line_39 == 'elseif (CAT==3)'
    assert SetAST[0].Program_Line_40 == 'set AHST = 22+AHSTtol'
    assert SetAST[0].Program_Line_41 == 'endif'
    assert SetAST[0].Program_Line_42 == 'else'
    assert SetAST[0].Program_Line_43 == 'if (CAT==1)'
    assert SetAST[0].Program_Line_44 == 'set AHST = 21+AHSTtol'
    assert SetAST[0].Program_Line_45 == 'elseif (CAT==2)'
    assert SetAST[0].Program_Line_46 == 'set AHST = 20+AHSTtol'
    assert SetAST[0].Program_Line_47 == 'elseif (CAT==3)'
    assert SetAST[0].Program_Line_48 == 'set AHST = 18+AHSTtol'
    assert SetAST[0].Program_Line_49 == 'endif'
    assert SetAST[0].Program_Line_50 == 'endif'
    assert SetAST[0].Program_Line_51 == 'endif'
    assert SetAST[0].Program_Line_52 == 'if (AdapStand == 1) && (ComfMod == 1) && (RMOT >= ACSTall) && (RMOT <= ACSTaul)'
    assert SetAST[0].Program_Line_53 == 'set ACST = RMOT*0.33+18.8+ACSToffset+ACSTtol'
    assert SetAST[0].Program_Line_54 == 'elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 7)'
    assert SetAST[0].Program_Line_55 == 'set ACST = 27+ACSTtol'
    assert SetAST[0].Program_Line_56 == 'elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 15)'
    assert SetAST[0].Program_Line_57 == 'set ACST = 50'
    assert SetAST[0].Program_Line_58 == 'elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 23)'
    assert SetAST[0].Program_Line_59 == 'set ACST = 25+ACSTtol'
    assert SetAST[0].Program_Line_60 == 'elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 24)'
    assert SetAST[0].Program_Line_61 == 'set ACST = 27+ACSTtol'
    assert SetAST[0].Program_Line_62 == 'endif'
    assert SetAST[0].Program_Line_63 == 'if (AdapStand == 1) && (ComfMod == 1) && (RMOT >= AHSTall) && (RMOT <= AHSTaul)'
    assert SetAST[0].Program_Line_64 == 'set AHST = RMOT*0.33+18.8+AHSToffset+AHSTtol'
    assert SetAST[0].Program_Line_65 == 'elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 7)'
    assert SetAST[0].Program_Line_66 == 'set AHST = 17+AHSTtol'
    assert SetAST[0].Program_Line_67 == 'elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 23)'
    assert SetAST[0].Program_Line_68 == 'set AHST = 20+AHSTtol'
    assert SetAST[0].Program_Line_69 == 'elseif (AdapStand == 1) && (ComfMod == 1) && (CurrentTime < 24)'
    assert SetAST[0].Program_Line_70 == 'set AHST = 17+AHSTtol'
    assert SetAST[0].Program_Line_71 == 'endif'
    assert SetAST[0].Program_Line_72 == 'if (AdapStand == 1) && (ComfMod == 2) && (RMOT >= ACSTall) && (RMOT <= ACSTaul)'
    assert SetAST[0].Program_Line_73 == 'set ACST = RMOT*0.33+18.8+ACSToffset+ACSTtol'
    assert SetAST[0].Program_Line_74 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < ACSTall) && (CAT==1)'
    assert SetAST[0].Program_Line_75 == 'set ACST = 25+ACSTtol'
    assert SetAST[0].Program_Line_76 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > ACSTaul) && (CAT==1)'
    assert SetAST[0].Program_Line_77 == 'set ACST = 25.5+ACSTtol'
    assert SetAST[0].Program_Line_78 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < ACSTall) && (CAT==2)'
    assert SetAST[0].Program_Line_79 == 'set ACST = 25+ACSTtol'
    assert SetAST[0].Program_Line_80 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > ACSTaul) && (CAT==2)'
    assert SetAST[0].Program_Line_81 == 'set ACST = 26+ACSTtol'
    assert SetAST[0].Program_Line_82 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < ACSTall) && (CAT==3)'
    assert SetAST[0].Program_Line_83 == 'set ACST = 25+ACSTtol'
    assert SetAST[0].Program_Line_84 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > ACSTaul) && (CAT==3)'
    assert SetAST[0].Program_Line_85 == 'set ACST = 27+ACSTtol'
    assert SetAST[0].Program_Line_86 == 'endif'
    assert SetAST[0].Program_Line_87 == 'if (AdapStand == 1) && (ComfMod == 2) && (RMOT >= AHSTall) && (RMOT <= AHSTaul)'
    assert SetAST[0].Program_Line_88 == 'set AHST = RMOT*0.33+18.8+AHSToffset+AHSTtol'
    assert SetAST[0].Program_Line_89 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < AHSTall) && (CAT==1)'
    assert SetAST[0].Program_Line_90 == 'set AHST = 21+AHSTtol'
    assert SetAST[0].Program_Line_91 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > AHSTaul) && (CAT==1)'
    assert SetAST[0].Program_Line_92 == 'set AHST = 23.5+AHSTtol'
    assert SetAST[0].Program_Line_93 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < AHSTall) && (CAT==2)'
    assert SetAST[0].Program_Line_94 == 'set AHST = 20+AHSTtol'
    assert SetAST[0].Program_Line_95 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > AHSTaul) && (CAT==2)'
    assert SetAST[0].Program_Line_96 == 'set AHST = 23+AHSTtol'
    assert SetAST[0].Program_Line_97 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT < AHSTall) && (CAT==3)'
    assert SetAST[0].Program_Line_98 == 'set AHST = 18+AHSTtol'
    assert SetAST[0].Program_Line_99 == 'elseif (AdapStand == 1) && (ComfMod == 2) && (RMOT > AHSTaul) && (CAT==3)'
    assert SetAST[0].Program_Line_100 == 'set AHST = 22+AHSTtol'
    assert SetAST[0].Program_Line_101 == 'endif'
    assert SetAST[0].Program_Line_102 == 'if (AdapStand == 1) && (ComfMod == 3) && (RMOT >= ACSTall) && (RMOT <= ACSTaul)'
    assert SetAST[0].Program_Line_103 == 'set ACST = RMOT*0.33+18.8+ACSToffset+ACSTtol'
    assert SetAST[0].Program_Line_104 == 'elseif (AdapStand == 1) && (ComfMod == 3) && (RMOT < ACSTall)'
    assert SetAST[0].Program_Line_105 == 'set ACST = ACSTall*0.33+18.8+ACSToffset+ACSTtol'
    assert SetAST[0].Program_Line_106 == 'elseif (AdapStand == 1) && (ComfMod == 3) && (RMOT > ACSTaul)'
    assert SetAST[0].Program_Line_107 == 'set ACST = ACSTaul*0.33+18.8+ACSToffset+ACSTtol'
    assert SetAST[0].Program_Line_108 == 'endif'
    assert SetAST[0].Program_Line_109 == 'if (AdapStand == 1) && (ComfMod == 3) && (RMOT >= AHSTall) && (RMOT <= AHSTaul)'
    assert SetAST[0].Program_Line_110 == 'set AHST = RMOT*0.33+18.8+AHSToffset+AHSTtol'
    assert SetAST[0].Program_Line_111 == 'elseif (AdapStand == 1) && (ComfMod == 3) && (RMOT < AHSTall)'
    assert SetAST[0].Program_Line_112 == 'set AHST = AHSTall*0.33+18.8+AHSToffset+AHSTtol'
    assert SetAST[0].Program_Line_113 == 'elseif (AdapStand == 1) && (ComfMod == 3) && (RMOT > AHSTaul)'
    assert SetAST[0].Program_Line_114 == 'set AHST = AHSTaul*0.33+18.8+AHSToffset+AHSTtol'
    assert SetAST[0].Program_Line_115 == 'endif'
    assert SetAST[0].Program_Line_116 == 'if (AdapStand == 2) && (ComfMod == 0)'
    assert SetAST[0].Program_Line_117 == 'if (DayOfYear >= 121) && (DayOfYear < 274)'
    assert SetAST[0].Program_Line_118 == 'if (CAT==80)'
    assert SetAST[0].Program_Line_119 == 'set ACST = 26.8+ACSTtol'
    assert SetAST[0].Program_Line_120 == 'elseif (CAT==90)'
    assert SetAST[0].Program_Line_121 == 'set ACST = 25.8+ACSTtol'
    assert SetAST[0].Program_Line_122 == 'endif'
    assert SetAST[0].Program_Line_123 == 'else'
    assert SetAST[0].Program_Line_124 == 'if (CAT==80)'
    assert SetAST[0].Program_Line_125 == 'set ACST = 22.9+ACSTtol'
    assert SetAST[0].Program_Line_126 == 'elseif (CAT==90)'
    assert SetAST[0].Program_Line_127 == 'set ACST = 23.9+ACSTtol'
    assert SetAST[0].Program_Line_128 == 'endif'
    assert SetAST[0].Program_Line_129 == 'endif'
    assert SetAST[0].Program_Line_130 == 'endif'
    assert SetAST[0].Program_Line_131 == 'if (AdapStand == 2) && (ComfMod == 0)'
    assert SetAST[0].Program_Line_132 == 'if (DayOfYear >= 121) && (DayOfYear < 274)'
    assert SetAST[0].Program_Line_133 == 'if (CAT==80)'
    assert SetAST[0].Program_Line_134 == 'set AHST = 24.6+AHSTtol'
    assert SetAST[0].Program_Line_135 == 'elseif (CAT==90)'
    assert SetAST[0].Program_Line_136 == 'set AHST = 23.6+AHSTtol'
    assert SetAST[0].Program_Line_137 == 'endif'
    assert SetAST[0].Program_Line_138 == 'else'
    assert SetAST[0].Program_Line_139 == 'if (CAT==80)'
    assert SetAST[0].Program_Line_140 == 'set AHST = 18.6+AHSTtol'
    assert SetAST[0].Program_Line_141 == 'elseif (CAT==90)'
    assert SetAST[0].Program_Line_142 == 'set AHST = 19.6+AHSTtol'
    assert SetAST[0].Program_Line_143 == 'endif'
    assert SetAST[0].Program_Line_144 == 'endif'
    assert SetAST[0].Program_Line_145 == 'endif'
    assert SetAST[0].Program_Line_146 == 'if (AdapStand == 2) && (ComfMod == 1) && (PMOT >= ACSTall) && (PMOT <= ACSTaul)'
    assert SetAST[0].Program_Line_147 == 'set ACST = PMOT*0.31+17.8+ACSToffset+ACSTtol'
    assert SetAST[0].Program_Line_148 == 'elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 7)'
    assert SetAST[0].Program_Line_149 == 'set ACST = 27+ACSTtol'
    assert SetAST[0].Program_Line_150 == 'elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 15)'
    assert SetAST[0].Program_Line_151 == 'set ACST = 50'
    assert SetAST[0].Program_Line_152 == 'elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 23)'
    assert SetAST[0].Program_Line_153 == 'set ACST = 25+ACSTtol'
    assert SetAST[0].Program_Line_154 == 'elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 24)'
    assert SetAST[0].Program_Line_155 == 'set ACST = 27+ACSTtol'
    assert SetAST[0].Program_Line_156 == 'endif'
    assert SetAST[0].Program_Line_157 == 'if (AdapStand == 2) && (ComfMod == 1) && (PMOT >= AHSTall) && (PMOT <= AHSTaul)'
    assert SetAST[0].Program_Line_158 == 'set AHST = PMOT*0.31+17.8+AHSToffset+AHSTtol'
    assert SetAST[0].Program_Line_159 == 'elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 7)'
    assert SetAST[0].Program_Line_160 == 'set AHST = 17+AHSTtol'
    assert SetAST[0].Program_Line_161 == 'elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 23)'
    assert SetAST[0].Program_Line_162 == 'set AHST = 20+AHSTtol'
    assert SetAST[0].Program_Line_163 == 'elseif (AdapStand == 2) && (ComfMod == 1) && (CurrentTime < 24)'
    assert SetAST[0].Program_Line_164 == 'set AHST = 17+AHSTtol'
    assert SetAST[0].Program_Line_165 == 'endif'
    assert SetAST[0].Program_Line_166 == 'if (AdapStand == 2) && (ComfMod == 2) && (PMOT >= ACSTall) && (PMOT <= ACSTaul)'
    assert SetAST[0].Program_Line_167 == 'set ACST = PMOT*0.31+17.8+ACSToffset+ACSTtol'
    assert SetAST[0].Program_Line_168 == 'elseif (AdapStand == 2) && (ComfMod == 2) && (PMOT < ACSTall)'
    assert SetAST[0].Program_Line_169 == 'set ACST = 23.9+ACSTtol'
    assert SetAST[0].Program_Line_170 == 'elseif (AdapStand == 2) && (ComfMod == 2) && (PMOT > ACSTaul)'
    assert SetAST[0].Program_Line_171 == 'set ACST = 26.8+ACSTtol'
    assert SetAST[0].Program_Line_172 == 'endif'
    assert SetAST[0].Program_Line_173 == 'if (AdapStand == 2) && (ComfMod == 2) && (PMOT >= AHSTall) && (PMOT <= AHSTaul)'
    assert SetAST[0].Program_Line_174 == 'set AHST = PMOT*0.31+17.8+AHSToffset+AHSTtol'
    assert SetAST[0].Program_Line_175 == 'elseif (AdapStand == 2) && (ComfMod == 2) && (PMOT < AHSTall)'
    assert SetAST[0].Program_Line_176 == 'set AHST = 19.6+AHSTtol'
    assert SetAST[0].Program_Line_177 == 'elseif (AdapStand == 2) && (ComfMod == 2) && (PMOT > AHSTaul)'
    assert SetAST[0].Program_Line_178 == 'set AHST = 23.6+AHSTtol'
    assert SetAST[0].Program_Line_179 == 'endif'
    assert SetAST[0].Program_Line_180 == 'if (AdapStand == 2) && (ComfMod == 3) && (PMOT >= ACSTall) && (PMOT <= ACSTaul)'
    assert SetAST[0].Program_Line_181 == 'set ACST = PMOT*0.31+17.8+ACSToffset+ACSTtol'
    assert SetAST[0].Program_Line_182 == 'elseif (AdapStand == 2) && (ComfMod == 3) && (PMOT < ACSTall)'
    assert SetAST[0].Program_Line_183 == 'set ACST = ACSTall*0.31+17.8+ACSToffset+ACSTtol'
    assert SetAST[0].Program_Line_184 == 'elseif (AdapStand == 2) && (ComfMod == 3) && (PMOT > ACSTaul)'
    assert SetAST[0].Program_Line_185 == 'set ACST = ACSTaul*0.31+17.8+ACSToffset+ACSTtol'
    assert SetAST[0].Program_Line_186 == 'endif'
    assert SetAST[0].Program_Line_187 == 'if (AdapStand == 2) && (ComfMod == 3) && (PMOT >= AHSTall) && (PMOT <= AHSTaul)'
    assert SetAST[0].Program_Line_188 == 'set AHST = PMOT*0.31+17.8+AHSToffset+AHSTtol'
    assert SetAST[0].Program_Line_189 == 'elseif (AdapStand == 2) && (ComfMod == 3) && (PMOT < AHSTall)'
    assert SetAST[0].Program_Line_190 == 'set AHST = AHSTall*0.31+17.8+AHSToffset+AHSTtol'
    assert SetAST[0].Program_Line_191 == 'elseif (AdapStand == 2) && (ComfMod == 3) && (PMOT > AHSTaul)'
    assert SetAST[0].Program_Line_192 == 'set AHST = AHSTaul*0.31+17.8+AHSToffset+AHSTtol'
    assert SetAST[0].Program_Line_193 == 'endif'
    
    SetASTnoTol = ([program
               for program
               in idf1.idfobjects['EnergyManagementSystem:Program']
               if program.Name == 'SetASTnoTol'])
    assert SetASTnoTol[0].Name == 'SetASTnoTol'
    assert SetASTnoTol[0].Program_Line_1 == 'set ACSTnoTol = ACST-ACSTtol'
    assert SetASTnoTol[0].Program_Line_2 == 'set AHSTnoTol = AHST-AHSTtol'

    for zonename in zonenames:
        CountHoursNoApp = ([program
                        for program
                        in idf1.idfobjects['EnergyManagementSystem:Program']
                        if program.Name == 'CountHoursNoApp_' + zonename])
        assert CountHoursNoApp[0].Name == 'CountHoursNoApp_' + zonename
        assert CountHoursNoApp[0].Program_Line_1 == 'if (' + zonename + '_OpT <= ACSTnoTol)'
        assert CountHoursNoApp[0].Program_Line_2 == 'if (' + zonename + '_OpT >= AHSTnoTol)'
        assert CountHoursNoApp[0].Program_Line_3 == 'set ComfHoursNoApp_' + zonename + '  = 1*ZoneTimeStep'
        assert CountHoursNoApp[0].Program_Line_4 == 'else'
        assert CountHoursNoApp[0].Program_Line_5 == 'set ComfHoursNoApp_' + zonename + ' = 0'
        assert CountHoursNoApp[0].Program_Line_6 == 'endif'
        assert CountHoursNoApp[0].Program_Line_7 == 'else'
        assert CountHoursNoApp[0].Program_Line_8 == 'set ComfHoursNoApp_' + zonename + ' = 0'
        assert CountHoursNoApp[0].Program_Line_9 == 'endif'


def test_addEMSPCMBase(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addEMSProgramsBase(verboseMode=False)
    IDFobject.addEMSPCMBase(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    programlist = ([program.Name for program in idf1.idfobjects['EnergyManagementSystem:Program']])

    for i in programlist:
        pcm = ([pcm for pcm in idf1.idfobjects['EnergyManagementSystem:ProgramCallingManager'] if pcm.Name == i])
        assert pcm[0].Name == i
        assert pcm[0].EnergyPlus_Model_Calling_Point == "BeginTimestepBeforePredictor"
        assert pcm[0].Program_Name_1 == i

def test_addEMSOutputVariableBase(IDFobject):
    from eppy.modeleditor import IDF
    
    IDFobject.addEMSOutputVariableBase(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')
    
    EMSOutputVariableComfTemp_dict = {
        'Comfort Temperature': 'ComfTemp',
        'Adaptive Cooling Setpoint Temperature': 'ACST',
        'Adaptive Heating Setpoint Temperature': 'AHST',
        'Adaptive Cooling Setpoint Temperature_No Tolerance': 'ACSTnoTol',
        'Adaptive Heating Setpoint Temperature_No Tolerance': 'AHSTnoTol',
        }
    
    for i in EMSOutputVariableComfTemp_dict:
        outputvariable = ([x
                           for x
                           in idf1.idfobjects['EnergyManagementSystem:OutputVariable']
                           if x.Name == i])
        assert outputvariable[0].Name == i
        assert outputvariable[0].EMS_Variable_Name == EMSOutputVariableComfTemp_dict[i]
        assert outputvariable[0].Type_of_Data_in_Variable == 'Averaged'
        assert outputvariable[0].Update_Frequency == 'ZoneTimestep'
        assert outputvariable[0].EMS_Program_or_Subroutine_Name == ''
        assert outputvariable[0].Units == 'C'

    EMSOutputVariableComfHours_dict = {
        'Comfortable Hours_No Applicability': 'ComfHoursNoApp',
        'Comfortable Hours': 'ComfHours',
        'Discomfortable Applicable Hot Hours': 'DiscomfAppHotHours',
        'Discomfortable Applicable Cold Hours': 'DiscomfAppColdHours',
        'Discomfortable Non Applicable Hot Hours': 'DiscomfNonAppHotHours',
        'Discomfortable Non Applicable Cold Hours': 'DiscomfNonAppColdHours',
        }

    zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in idf1.idfobjects['ZONE']])])
    for i in EMSOutputVariableComfHours_dict:
        for zonename in zonenames:
            outputvariable = ([x
                               for x
                               in idf1.idfobjects['EnergyManagementSystem:OutputVariable']
                               if x.Name == i + '_' + zonename + ' (summed)'])
            assert outputvariable[0].Name == i + '_' + zonename + ' (summed)'
            assert outputvariable[0].EMS_Variable_Name == EMSOutputVariableComfHours_dict[i]+'_' + zonename
            assert outputvariable[0].Type_of_Data_in_Variable == 'Summed'
            assert outputvariable[0].Update_Frequency == 'ZoneTimestep'
            assert outputvariable[0].EMS_Program_or_Subroutine_Name == ''
            assert outputvariable[0].Units == 'H'


def test_addOutputVariablesTimestep(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addOutputVariablesSingleZone(verboseMode=False)
    IDFobject.addOutputVariablesTimestep(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    fulloutputlist = ([output for output in idf1.idfobjects['Output:Variable']])
    outputlist = ([output.Variable_Name for output in idf1.idfobjects['Output:Variable']])

    for i in range(len(outputlist)):
        outputvariable = ([x
                           for x
                           in idf1.idfobjects['Output:Variable']
                           if x.Key_Value == fulloutputlist[i].Key_Value
                           and x.Variable_Name == fulloutputlist[i].Variable_Name
                           and x.Reporting_Frequency == 'Timestep'])
        assert outputvariable[0].Key_Value == fulloutputlist[i].Key_Value
        assert outputvariable[0].Variable_Name == fulloutputlist[i].Variable_Name
        assert outputvariable[0].Reporting_Frequency == 'Timestep'
        assert outputvariable[0].Schedule_Name == fulloutputlist[i].Schedule_Name


def test_addSimplifiedOutputVariables(IDFobject):
    from eppy.modeleditor import IDF

    IDFobject.addSimplifiedOutputVariables(verboseMode=False)
    IDFobject.saveaccim(verboseMode=False)
    idf1 = IDF('TestModel_SingleZone_pymod.idf')

    EnvironmentalImpactFactorslist = ([output for output in idf1.idfobjects['Output:EnvironmentalImpactFactors']])
    outputmeterlist = ([output for output in idf1.idfobjects['Output:Meter']])
    alloutputs = ([output for output in idf1.idfobjects['Output:Variable']])

    assert len(EnvironmentalImpactFactorslist) == 0
    assert len(outputmeterlist) == 0

    addittionaloutputs = ['Zone Thermostat Operative Temperature',
                          'VRF Heat Pump Cooling Electricity Energy',
                          'VRF Heat Pump Heating Electricity Energy',
                          'Facility Total HVAC Electric Demand Power',
                          'Facility Total HVAC Electricity Demand Rate'
                          ]
    for addittionaloutput in addittionaloutputs:
        outputvariable = ([x
                           for x
                           in idf1.idfobjects['Output:Variable']
                           if x.Variable_Name == addittionaloutput])
        assert outputvariable[0].Key_Value == '*'
        assert outputvariable[0].Variable_Name == addittionaloutput
        assert outputvariable[0].Reporting_Frequency == 'Hourly'
        assert outputvariable[0].Schedule_Name == ''



# -*- coding: utf-8 -*-
"""
Created on Sun Feb 28 17:03:48 2021

@author: Daniel
"""

from eppy import modeleditor
from eppy.modeleditor import IDF

import pytest

from accim.sim import accim_Base_EMS

# from accim.sim import accis
# accis.addAccis('sz', 'standard', 'ep94', [1], [3], [3], 0.1, 0.1, 0.1)

def lookfor(self, name, objectType):
    """
    Look for name in objectType list.
    """
    listToSearchIn = ([x.Name for x in self.idf0.idfobjects[objectType]])

    if name in listToSearchIn:
        return True
    else:
        return False


def test_addEMSProgramsBase(self, filename_temp):
    from accim.sim import accim_Main
    
    from eppy import modeleditor
    from eppy.modeleditor import IDF

    iddfile = 'C:/EnergyPlusV9-4-0/Energy+.idd'
    IDF.setiddname(iddfile)

    fname1 = filename_temp+'.idf'
    self.idf0 = IDF(fname1)
    self.idf0.savecopy(filename_temp+'_pymod.idf')

    self.filename = filename_temp+'_pymod'
    fname1 = self.filename+'.idf'
    self.idf1 = IDF(fname1)
    self.filename = filename_temp+'_pymod'

    print(self.filename)

    self.zonenames_orig = ([zone.Name for zone in self.idf1.idfobjects['ZONE']])
    # print(self.zonenames_orig)

    self.zonenames = ([sub.replace(':', '_') for sub in ([zone.Name for zone in self.idf1.idfobjects['ZONE']])])
    # print(self.zonenames)
    
    z = accim_Main.accimobj_SingleZone_Ep94('TestModel_SingleZone')
    z.addEMSProgramsBase()

    programlist = ([program.Name for program in self.idf1.idfobjects['EnergyManagementSystem:Program']])
    print(programlist)
    
    
    
    # lookfor('SetComfTemp','EnergyManagementSystem:Program')
    
    # listToSearchIn = ([x.Name for x in self.idf1.idfobjects['EnergyManagementSystem:Program']])

    # if 'SetComfTemp' in listToSearchIn:
    #     return True
    # else:
    #     return False
    
test_addEMSProgramsBase()